@@ -11,6 +11,11 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
+#ifndef INET_ADDRSTRLEN
+#	define INET_ADDRSTRLEN 16
+#endif
+
+
 /*
 ** This is an IPv4 only inet_ntop(3) replacement function.
 */
