@@ -1,6 +1,6 @@
 /*
 ** pork.c - main function.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -33,6 +33,7 @@
 #include <pork_util.h>
 #include <pork_list.h>
 #include <pork_set.h>
+#include <pork_imsg.h>
 #include <pork_imwindow.h>
 #include <pork_acct.h>
 #include <pork_misc.h>
@@ -40,7 +41,7 @@
 #include <pork_bind.h>
 #include <pork_events.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_status.h>
 #include <pork_alias.h>
 #include <pork_opt.h>
@@ -56,12 +57,6 @@
 #include <pork_inet.h>
 #include <pork_missing.h>
 
-extern int aim_proto_init(struct pork_proto *proto);
-
-#ifdef IRC_SUPPORT
-extern int irc_proto_init(struct pork_proto *proto);
-#endif
-
 struct screen screen;
 
 /*
@@ -137,15 +132,15 @@ void keyboard_input(int fd __notused,
 		return;
 
 	time(&acct->last_input);
+	bind_exec(imwindow->active_binds, key);
 
 	if (acct->connected && acct->marked_idle && opt_get_bool(OPT_REPORT_IDLE)) {
 		if (acct->proto->set_idle_time != NULL)
 			acct->proto->set_idle_time(acct, 0);
 		acct->marked_idle = 0;
-		screen_status_msg_ts("%s is no longer marked idle", acct->username);
+		screen_win_msg(cur_window(), 1, 1, 0,
+			MSG_TYPE_UNIDLE, "%s is no longer marked idle", acct->username);
 	}
-
-	bind_exec(imwindow->active_binds, key);
 }
 
 int main(int argc, char **argv) {
@@ -156,8 +151,6 @@ int main(int argc, char **argv) {
 	time_t timer_last_run;
 	time_t status_last_update = 0;
 
-	proto_init();
-
 	pw = getpwuid(getuid());
 	if (pw == NULL) {
 		fprintf(stderr, "Fatal: Can't get your user info.\n");
@@ -177,12 +170,7 @@ int main(int argc, char **argv) {
 		exit(-1);
 	}
 
-	proto_new(PROTO_AIM, "AIM", aim_proto_init);
-
-#ifdef IRC_SUPPORT
-	proto_new(PROTO_IRC, "IRC", irc_proto_init);
-#endif
-
+	proto_init();
 	color_init();
 	pork_io_init();
 	perl_init();
@@ -202,6 +190,7 @@ int main(int argc, char **argv) {
 	bind_init(&screen.binds);
 	bind_set_handlers(&screen.binds.main, binding_run, binding_insert);
 	bind_set_handlers(&screen.binds.blist, binding_run, NULL);
+
 	alias_init(&screen.alias_hash);
 	event_init(&screen.events);
 
