@@ -1,6 +1,6 @@
 /*
 ** pork.h - pork main include file.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -11,9 +11,11 @@
 #define __PORK_H
 
 #ifdef HAVE___ATTRIBUTE__
+	#define __used		__attribute__((used))
 	#define __notused	__attribute__((unused))
 	#define __format(x) __attribute__((format x ))
 #else
+	#define __used
 	#define __notused
 	#define __format(x)
 #endif
