@@ -1,6 +1,6 @@
 /*
 ** pork_chat.c
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -27,7 +27,8 @@
 #include <pork_proto.h>
 #include <pork_format.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
+#include <pork_imsg.h>
 #include <pork_imwindow.h>
 #include <pork_color.h>
 #include <pork_chat.h>
@@ -78,10 +79,15 @@ int chat_send_msg(	struct pork_acct *acct,
 	{
 		if (acct->proto->chat_send(acct, chat, target, msg) != -1) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_SEND, buf, sizeof(buf),
-				acct, chat, target, msg);
-			imwindow_send_msg(chat->win, acct, buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_SEND, buf, sizeof(buf),
+					acct, chat, target, msg);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_MSG_SEND);
+			imwindow_send_msg(chat->win);
 		}
 	}
 
@@ -101,10 +107,15 @@ int chat_send_notice(	struct pork_acct *acct,
 	{
 		if (acct->proto->chat_send_notice(acct, chat, target, msg) != -1) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_SEND_NOTICE, buf, sizeof(buf),
+			ret = fill_format_str(OPT_FORMAT_CHAT_SEND_NOTICE, buf, sizeof(buf),
 				acct, chat, target, msg);
-			imwindow_send_msg(chat->win, acct, buf);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_NOTICE_SEND);
+			imwindow_send_msg(chat->win);
 		}
 	}
 
@@ -124,10 +135,15 @@ int chat_send_action(	struct pork_acct *acct,
 	{
 		if (acct->proto->chat_action(acct, chat, target, msg) != -1) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_SEND_ACTION, buf, sizeof(buf),
-				acct, chat, target, msg);
-			imwindow_send_msg(chat->win, acct, buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_SEND_ACTION, buf, sizeof(buf),
+					acct, chat, target, msg);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_ACTION_SEND);
+			imwindow_send_msg(chat->win);
 		} else
 			return (-1);
 	}
@@ -147,10 +163,15 @@ int chat_recv_action(	struct pork_acct *acct,
 	{
 		if (!chat_user_is_ignored(acct, chat, user)) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_RECV_ACTION, buf, sizeof(buf), acct,
-				chat, dest, user, userhost, msg);
-			imwindow_recv_msg(chat->win, acct, buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_RECV_ACTION, buf,
+					sizeof(buf), acct, chat, dest, user, userhost, msg);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_ACTION_RECV);
+			imwindow_recv_msg(chat->win);
 		}
 	}
 
@@ -169,10 +190,15 @@ int chat_recv_msg(	struct pork_acct *acct,
 	{
 		if (!chat_user_is_ignored(acct, chat, user)) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_RECV, buf, sizeof(buf), acct,
-				chat, dest, user, userhost, msg);
-			imwindow_recv_msg(chat->win, acct, buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_RECV, buf, sizeof(buf), acct,
+					chat, dest, user, userhost, msg);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_MSG_RECV);
+			imwindow_recv_msg(chat->win);
 		}
 	}
 
@@ -191,10 +217,15 @@ int chat_recv_notice(	struct pork_acct *acct,
 	{
 		if (!chat_user_is_ignored(acct, chat, user)) {
 			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_RECV_NOTICE, buf, sizeof(buf), acct,
-				chat, dest, user, userhost, msg);
-			imwindow_recv_msg(chat->win, acct, buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_RECV_NOTICE, buf,
+					sizeof(buf), acct, chat, dest, user, userhost, msg);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_NOTICE_RECV);
+			imwindow_recv_msg(chat->win);
 		}
 	}
 
@@ -204,7 +235,8 @@ int chat_recv_notice(	struct pork_acct *acct,
 int chat_ignore(struct pork_acct *acct, char *chat_name, char *user) {
 	struct chat_user *chat_user;
 	struct chatroom *chat;
-	char buf[512];
+	char buf[4096];
+	int ret;
 
 	chat = chat_find(acct, chat_name);
 	if (chat == NULL)
@@ -223,17 +255,21 @@ int chat_ignore(struct pork_acct *acct, char *chat_name, char *user) {
 			return (-1);
 	}
 
-	fill_format_str(OPT_FORMAT_CHAT_IGNORE, buf, sizeof(buf), acct, chat,
-		chat->title, acct->username, user, NULL);
+	ret = fill_format_str(OPT_FORMAT_CHAT_IGNORE, buf, sizeof(buf),
+			acct, chat, chat->title, acct->username, user, NULL);
+	if (ret < 1)
+		return (-1);
+	screen_print_str(chat->win, buf, (size_t) ret,
+		MSG_TYPE_CHAT_STATUS);
 
-	screen_win_msg(cur_window(), 1, 1, 1, "%s", buf);
 	return (0);
 }
 
 int chat_unignore(struct pork_acct *acct, char *chat_name, char *user) {
 	struct chat_user *chat_user;
 	struct chatroom *chat;
-	char buf[512];
+	char buf[4096];
+	int ret;
 
 	chat = chat_find(acct, chat_name);
 	if (chat == NULL)
@@ -252,10 +288,13 @@ int chat_unignore(struct pork_acct *acct, char *chat_name, char *user) {
 			return (-1);
 	}
 
-	fill_format_str(OPT_FORMAT_CHAT_UNIGNORE, buf, sizeof(buf), acct, chat,
-		chat->title, acct->username, user, NULL);
+	ret = fill_format_str(OPT_FORMAT_CHAT_UNIGNORE, buf, sizeof(buf), acct,
+			chat, chat->title, acct->username, user, NULL);
+	if (ret < 1)
+		return (-1);
+	screen_print_str(chat->win, buf, (size_t) ret,
+		MSG_TYPE_CHAT_STATUS);
 
-	screen_win_msg(cur_window(), 1, 1, 1, "%s", buf);
 	return (0);
 }
 
@@ -395,27 +434,29 @@ int chat_user_kicked(	struct pork_acct *acct,
 						char *kicker,
 						char *reason)
 {
-	char buf[512];
-	int silent = 0;
-
-	if (event_generate(acct->events, EVENT_RECV_CHAT_KICK,
-			chat->title, kicked, kicker, reason, acct->refnum))
+	if (!event_generate(acct->events, EVENT_RECV_CHAT_KICK,
+		chat->title, kicked, kicker, reason, acct->refnum))
 	{
-		silent = 1;
-	}
+		char buf[4096];
+		int ret;
 
-	if (!silent) {
-		fill_format_str(OPT_FORMAT_CHAT_KICK, buf, sizeof(buf), acct, chat,
-			chat->title, kicker, kicked, reason);
-
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_KICK, buf, sizeof(buf), acct,
+				chat, chat->title, kicker, kicked, reason);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	chat_user_left(acct, chat, kicked, 1);
 	return (0);
 }
 
-int chat_forced_leave(struct pork_acct *acct, char *chat_name, char *person, char *reason) {
+int chat_forced_leave(	struct pork_acct *acct,
+						char *chat_name,
+						char *target,
+						char *reason)
+{
 	struct chatroom *chat;
 
 	chat = chat_find(acct, chat_name);
@@ -424,7 +465,7 @@ int chat_forced_leave(struct pork_acct *acct, char *chat_name, char *person, cha
 		return (-1);
 	}
 
-	chat_user_kicked(acct, chat, acct->username, person, reason);
+	chat_user_kicked(acct, chat, acct->username, target, reason);
 
 	if (opt_get_bool(OPT_AUTO_REJOIN))
 		return (chat_rejoin(acct, chat));
@@ -446,15 +487,18 @@ int chat_invite(struct pork_acct *acct, char *chat_name, char *user, char *msg)
 	if (!event_generate(acct->events, EVENT_SEND_CHAT_INVITE, chat->title,
 		user, msg, acct->refnum))
 	{
-		char buf[512];
+		char buf[4096];
+		int ret;
 
 		if (acct->proto->chat_invite(acct, chat, user, msg) == -1)
 			return (-1);
 
-		fill_format_str(OPT_FORMAT_CHAT_INVITE, buf, sizeof(buf), acct, chat,
-			chat->title, acct->username, user, msg);
-
-		screen_win_msg(cur_window(), 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_INVITE, buf, sizeof(buf), acct,
+				chat, chat->title, acct->username, user, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	return (0);
@@ -472,15 +516,18 @@ void chat_list(struct pork_acct *acct) {
 
 	cur = acct->chat_list;
 	if (cur == NULL) {
-		screen_output("%s is not a member of any chat rooms", acct->username);
+		screen_cmd_output("%s is not a member of any chat rooms",
+			acct->username);
 		return;
 	}
 
-	screen_output("%s is joined to the following chat rooms", acct->username);
+	screen_cmd_output("%s is joined to the following chat rooms",
+		acct->username);
+
 	do {
 		struct chatroom *chat = cur->data;
 
-		screen_output("  %s in window refnum %u",
+		screen_cmd_output("  %s in window refnum %u",
 			chat->title_quoted, chat->win->refnum);
 
 		cur = cur->next;
@@ -500,12 +547,15 @@ int chat_free(struct pork_acct *acct, struct chatroom *chat, int silent) {
 
 	if (chat->win != NULL) {
 		if (!silent) {
-			char buf[512];
+			char buf[4096];
+			int ret;
 
-			fill_format_str(OPT_FORMAT_CHAT_LEAVE, buf, sizeof(buf), acct, chat,
-				chat->title, acct->username, NULL, NULL);
-
-			screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+			ret = fill_format_str(OPT_FORMAT_CHAT_LEAVE, buf, sizeof(buf), acct,
+					chat, chat->title, acct->username, NULL, NULL);
+			if (ret < 1)
+				return (-1);
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_STATUS);
 		}
 
 		chat->win->data = NULL;
@@ -536,24 +586,31 @@ int chat_got_invite(struct pork_acct *acct,
 	if (!event_generate(acct->events, EVENT_RECV_CHAT_INVITE,
 			chat_name, user, userhost, message, acct->refnum))
 	{
-		char buf[512];
+		char buf[4096];
+		int ret;
 
-		fill_format_str(OPT_FORMAT_CHAT_INVITE, buf, sizeof(buf), acct, NULL,
-			chat_name, user, acct->username, message);
-
-		screen_win_msg(cur_window(), 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_INVITE, buf, sizeof(buf),
+			acct, NULL, chat_name, user, acct->username, message);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	return (0);
 }
 
 int chat_created(struct pork_acct *acct, struct chatroom *chat) {
-	char buf[512];
+	char buf[4096];
+	int ret;
 
-	fill_format_str(OPT_FORMAT_CHAT_CREATE, buf, sizeof(buf), acct, chat,
-		chat->title, acct->username, NULL, NULL);
+	ret = fill_format_str(OPT_FORMAT_CHAT_CREATE, buf, sizeof(buf), acct, chat,
+			chat->title, acct->username, NULL, NULL);
+	if (ret < 1)
+		return (-1);
+	screen_print_str(chat->win, buf, (size_t) ret,
+		MSG_TYPE_CHAT_STATUS);
 
-	screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
 	return (0);
 }
 
@@ -563,7 +620,7 @@ struct chat_user *chat_user_joined(	struct pork_acct *acct,
 									char *host,
 									int silent)
 {
-	char buf[512];
+	char buf[4096];
 	struct chat_user *chat_user;
 
 	if (chat_find_user(acct, chat, user) != NULL) {
@@ -572,7 +629,6 @@ struct chat_user *chat_user_joined(	struct pork_acct *acct,
 	}
 
 	chat->num_users++;
-
 	acct->proto->normalize(buf, user, sizeof(buf));
 
 	chat_user = xcalloc(1, sizeof(*chat_user));
@@ -591,10 +647,14 @@ struct chat_user *chat_user_joined(	struct pork_acct *acct,
 	}
 
 	if (!silent) {
-		fill_format_str(OPT_FORMAT_CHAT_JOIN, buf, sizeof(buf), acct, chat,
-			chat->title, user, NULL, NULL);
+		int ret;
 
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_JOIN, buf, sizeof(buf), acct,
+			chat, chat->title, user, NULL, NULL);
+		if (ret > 0) {
+			screen_print_str(chat->win, buf, (size_t) ret,
+				MSG_TYPE_CHAT_STATUS);
+		}
 	}
 
 	return (chat_user);
@@ -649,12 +709,15 @@ int chat_user_left(	struct pork_acct *acct,
 	}
 
 	if (!silent) {
-		char buf[512];
+		char buf[4096];
+		int ret;
 
-		fill_format_str(OPT_FORMAT_CHAT_LEAVE, buf, sizeof(buf), acct, chat,
-			chat->title, user, NULL, NULL);
-
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_LEAVE, buf, sizeof(buf), acct,
+				chat, chat->title, user, NULL, NULL);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	if (node != NULL) {
@@ -679,12 +742,15 @@ int chat_user_quit(	struct pork_acct *acct,
 	if (!event_generate(acct->events, EVENT_RECV_CHAT_QUIT,
 			chat->title, user->name, msg, acct->refnum))
 	{
-		char buf[512];
+		char buf[4096];
+		int ret;
 
-		fill_format_str(OPT_FORMAT_CHAT_QUIT, buf, sizeof(buf), acct, chat,
-			chat->title, user->name, NULL, msg);
-
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+		ret = fill_format_str(OPT_FORMAT_CHAT_QUIT, buf, sizeof(buf), acct,
+				chat, chat->title, user->name, NULL, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	return (chat_user_left(acct, chat, user->name, 1));
@@ -716,12 +782,37 @@ int chat_got_topic(	struct pork_acct *acct,
 	if (!event_generate(acct->events, EVENT_RECV_CHAT_TOPIC,
 		chat->title, set_by, topic, acct->refnum))
 	{
-		char buf[512];
+		char buf[4096];
+		int ret;
 
-		fill_format_str(OPT_FORMAT_CHAT_TOPIC, buf, sizeof(buf), acct, chat,
-			chat->title, set_by, NULL, topic);
+		ret = fill_format_str(OPT_FORMAT_CHAT_TOPIC, buf, sizeof(buf),
+				acct, chat, chat->title, set_by, NULL, topic);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
+	}
 
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
+	return (0);
+}
+
+int chat_got_mode(	struct pork_acct *acct,
+					struct chatroom *chat,
+					char *user,
+					char *mode)
+{
+	if (!event_generate(acct->events, EVENT_RECV_CHAT_MODE,
+			chat->title, user, mode, acct->refnum))
+	{
+		char buf[4096];
+		int ret;
+
+		ret = fill_format_str(OPT_FORMAT_CHAT_MODE, buf, sizeof(buf), acct,
+			chat, chat->title, user, NULL, mode);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(chat->win, buf, (size_t) ret,
+			MSG_TYPE_CHAT_STATUS);
 	}
 
 	return (0);
@@ -761,25 +852,6 @@ int chat_rejoin(struct pork_acct *acct, struct chatroom *chat) {
 	return (acct->proto->chat_rejoin(acct, chat));
 }
 
-int chat_got_mode(	struct pork_acct *acct,
-					struct chatroom *chat,
-					char *user,
-					char *mode)
-{
-	if (!event_generate(acct->events, EVENT_RECV_CHAT_MODE,
-			chat->title, user, mode, acct->refnum))
-	{
-		char buf[512];
-
-		fill_format_str(OPT_FORMAT_CHAT_MODE, buf, sizeof(buf), acct, chat,
-			chat->title, user, NULL, mode);
-
-		screen_win_msg(chat->win, 1, 1, 1, "%s", buf);
-	}
-
-	return (0);
-}
-
 int chat_rejoin_all(struct pork_acct *acct) {
 	dlist_t *cur;
 
@@ -791,3 +863,38 @@ int chat_rejoin_all(struct pork_acct *acct) {
 
 	return (0);
 }
+
+int chat_nick_change(struct pork_acct *acct, char *old, char *new_nick) {
+	dlist_t *cur;
+
+	cur = acct->chat_list;
+	while (cur != NULL) {
+		struct chat_user *user;
+		struct chatroom *chat = cur->data;
+
+		user = chat_find_user(acct, chat, old);
+		if (user != NULL) {
+			char buf[4096];
+			int ret;
+
+			ret = fill_format_str(OPT_FORMAT_CHAT_NICK, buf, sizeof(buf),
+					acct, chat, chat->title, old, new_nick, NULL);
+			if (ret > 0) {
+				screen_print_str(chat->win, buf, (size_t) ret,
+					MSG_TYPE_CHAT_STATUS);
+			}
+
+			free(user->name);
+			free(user->nname);
+
+			user->name = xstrdup(new_nick);
+
+			acct->proto->normalize(buf, new_nick, sizeof(buf));
+			user->nname = xstrdup(buf);
+		}
+
+		cur = cur->next;
+	}
+
+	return (0);
+}
