@@ -1,7 +1,7 @@
 /*
 ** pork_color.c - color functions.
 ** Copyright (C) 2002-2004 Amber Adams <amber@ojnk.net>
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -151,58 +151,22 @@ int color_parse_code(const char *code, attr_t *attr) {
 }
 
 static inline int color_get_code(char code) {
-	switch (tolower(code)) {
-		case 'd':
-			return (0);
+	static const char codes[] = { 4, 6, 0, -1, -1, 2, -1, -1, -1, -1, -1, 5, -1, -1, 5, -1, 1, -1, -1, -1, -1, 7, -1, 3 };
 
-		case 'r':
-			return (1);
+	code = tolower(code);
+	if (code < 'b' || code > 'y')
+		return (-1);
 
-		case 'g':
-			return (2);
-
-		case 'y':
-			return (3);
-
-		case 'b':
-			return (4);
-
-		case 'm':
-		case 'p':
-			return (5);
-
-		case 'c':
-			return (6);
-
-		case 'w':
-			return (7);
-	}
-
-	return (-1);
+	return ((int) codes[code - 'b']);
 }
 
 static inline int color_get_highlighting(char code) {
-	switch (code) {
-		case '1':
-			return (A_BOLD);
+	static const int colors[] = { A_BOLD, A_REVERSE, A_UNDERLINE, A_BLINK, A_STANDOUT, A_DIM };
 
-		case '2':
-			return (A_REVERSE);
+	if (code < '1' || code > '6')
+		return (-1);
 
-		case '3':
-			return (A_UNDERLINE);
-
-		case '4':
-			return (A_BLINK);
-
-		case '5':
-			return (A_STANDOUT);
-
-		case '6':
-			return (A_DIM);
-	}
-
-	return (-1);
+	return (colors[code - '1']);
 }
 
 int color_get_str(attr_t attr, char *buf, size_t len) {
