@@ -1,6 +1,6 @@
 /*
 ** pork_command.c - interface to commands typed by the user
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 ** Copyright (C) 2002-2004 Amber Adams <amber@ojnk.net>
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <pork_list.h>
 #include <pork_buddy.h>
 #include <pork_set.h>
+#include <pork_imsg.h>
 #include <pork_imwindow.h>
 #include <pork_buddy_list.h>
 #include <pork_proto.h>
@@ -40,7 +41,7 @@
 #include <pork_bind.h>
 #include <pork_events.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_alias.h>
 #include <pork_chat.h>
 #include <pork_conf.h>
@@ -70,7 +71,7 @@ static int run_one_command(char *str, u_int32_t set);
 ** The / command set.
 */
 
-static const struct command command[] = {
+static struct command command[] = {
 	{ "",			cmd_send			},
 	{ "acct",		cmd_acct			},
 	{ "alias",		cmd_alias			},
@@ -123,7 +124,7 @@ static const struct command command[] = {
 ** /input commands
 */
 
-static const struct command input_command[] = {
+static struct command input_command[] = {
 	{ "backspace",				cmd_input_bkspace			},
 	{ "clear",					cmd_input_clear				},
 	{ "clear_next_word",		cmd_input_clear_next		},
@@ -165,10 +166,10 @@ USER_COMMAND(cmd_input_clear_to_end) {
 }
 
 USER_COMMAND(cmd_input_focus_next) {
-	struct imwindow *imwindow = cur_window();
+	struct imwindow *win = cur_window();
 
-	imwindow_switch_focus(imwindow);
-	imwindow_blist_draw(imwindow);
+	imwindow_switch_focus(win);
+	imwindow_blist_draw(win);
 }
 
 USER_COMMAND(cmd_input_clear_to_start) {
@@ -199,12 +200,12 @@ USER_COMMAND(cmd_input_prev_word) {
 USER_COMMAND(cmd_input_prompt) {
 	if (args == NULL) {
 		input_set_prompt(cur_window()->input, NULL);
-		screen_status_msg("Input prompt cleared");
+		screen_cmd_output("Input prompt cleared");
 	} else {
 		if (input_set_prompt(cur_window()->input, args) == -1)
 			screen_err_msg("The requested prompt is too long for the screen");
 		else {
-			screen_status_msg("Input prompt set to %s", args);
+			screen_cmd_output("Input prompt set to %s", args);
 		}
 	}
 }
@@ -269,7 +270,7 @@ USER_COMMAND(cmd_input_start) {
 ** /scroll commands
 */
 
-static const struct command scroll_command[] = {
+static struct command scroll_command[] = {
 	{ "by",				cmd_scroll_by			},
 	{ "down",			cmd_scroll_down			},
 	{ "end",			cmd_scroll_end			},
@@ -286,7 +287,7 @@ USER_COMMAND(cmd_scroll_by) {
 		return;
 
 	if (str_to_int(args, &lines) != 0) {
-		screen_err_msg("Bad number of lines: %s", args);
+		screen_err_msg("Invalid number of lines: %s", args);
 		return;
 	}
 
@@ -321,7 +322,7 @@ USER_COMMAND(cmd_scroll_up) {
 ** /file commands.
 */
 
-static const struct command file_command[] = {
+static struct command file_command[] = {
 	{ "abort",				cmd_file_cancel		},
 	{ "cancel",				cmd_file_cancel 	},
 	{ "get",				cmd_file_get		},
@@ -334,10 +335,30 @@ USER_COMMAND(cmd_file_cancel) {
 	struct pork_acct *acct = cur_window()->owner;
 	u_int32_t refnum;
 	struct file_transfer *xfer;
+	char *refnum_str;
 
-	if (args == NULL)
+	if (args == NULL || blank_str(args))
 		return;
 
+	refnum_str = strsep(&args, " ");
+	if (refnum_str == NULL)
+		return;
+
+	if (!strcasecmp(refnum_str, "all")) {
+		transfer_abort_all(acct, TRANSFER_DIR_ANY);
+		return;
+	}
+
+	if (!strcasecmp(refnum_str, "send")) {
+		transfer_abort_all(acct, TRANSFER_DIR_SEND);
+		return;
+	}
+
+	if (!strcasecmp(refnum_str, "recv") || !strcasecmp(refnum_str, "receive")) {
+		transfer_abort_all(acct, TRANSFER_DIR_RECV);
+		return;
+	}
+
 	if (str_to_uint(args, &refnum) != 0) {
 		screen_err_msg("Invalid file transfer refnum: %s", args);
 		return;
@@ -362,7 +383,7 @@ USER_COMMAND(cmd_file_list) {
 		while (cur != NULL) {
 			struct file_transfer *xfer = cur->data;
 
-			screen_output("%u: %s %s [%s: %u/%u (%.02f%%) - %.04f KB/s]",
+			screen_cmd_output("%u: %s %s [%s: %u/%u (%.02f%%) - %.04f KB/s]",
 				xfer->refnum,
 				xfer->peer_username, xfer->fname_local,
 				transfer_status_str(xfer),
@@ -381,13 +402,18 @@ USER_COMMAND(cmd_file_get) {
 	struct pork_acct *acct = cur_window()->owner;
 	struct file_transfer *xfer;
 
-	if (args == NULL)
+	if (args == NULL || blank_str(args))
 		return;
 
 	refnum_str = strsep(&args, " ");
 	if (refnum_str == NULL)
 		return;
 
+	if (!strcasecmp(refnum_str, "all")) {
+		transfer_get_all(acct);
+		return;
+	}
+
 	if (str_to_uint(refnum_str, &refnum) != 0) {
 		screen_err_msg("Invalid file transfer refnum: %s", refnum_str);
 		return;
@@ -427,7 +453,7 @@ USER_COMMAND(cmd_file_send) {
 ** /win commands.
 */
 
-static const struct command window_command[] = {
+static struct command window_command[] = {
 	{ "bind",				cmd_win_bind		},
 	{ "bind_next",			cmd_win_bind_next	},
 	{ "clear",				cmd_win_clear		},
@@ -454,10 +480,10 @@ USER_COMMAND(cmd_win_bind) {
 
 	if (args == NULL || blank_str(args)) {
 		if (imwindow->owner != NULL && imwindow->owner->username != NULL) {
-			screen_output("This window is bound to account %s [refnum %u]",
+			screen_cmd_output("This window is bound to account %s [refnum %u]",
 				imwindow->owner->username, imwindow->owner->refnum);
 		} else
-			screen_output("This window is bound to no account");
+			screen_cmd_output("This window is bound to no account");
 
 		return;
 	}
@@ -474,7 +500,7 @@ USER_COMMAND(cmd_win_bind) {
 		else
 			screen_err_msg("Account %s isn't signed on", args);
 	} else {
-		screen_status_msg("This window is now bound to account %s [refnum %u]",
+		screen_cmd_output("This window is now bound to account %s [refnum %u]",
 			imwindow->owner->username, imwindow->owner->refnum);
 	}
 }
@@ -493,13 +519,14 @@ USER_COMMAND(cmd_win_close) {
 }
 
 USER_COMMAND(cmd_win_dump) {
-	char buf[4096];
+	if (args == NULL || blank_str(args)) {
+		screen_err_msg("No output file specified");
+	} else {
+		char buf[4096];
 
-	if (args == NULL || blank_str(args))
-		return;
-
-	expand_path(args, buf, sizeof(buf));
-	imwindow_dump_buffer(cur_window(), buf);
+		expand_path(args, buf, sizeof(buf));
+		imwindow_dump_buffer(cur_window(), buf);
+	}
 }
 
 USER_COMMAND(cmd_win_erase) {
@@ -532,14 +559,14 @@ USER_COMMAND(cmd_win_list) {
 	dlist_t *cur;
 	static const char *win_types[] = { "Status", "Conv", "Chat" };
 
-	screen_output("Window List:");
+	screen_cmd_output("Window List:");
 
-	screen_output("REFNUM\t\tNAME\t\tTYPE\t\tTARGET");
+	screen_cmd_output("REFNUM\t\tNAME\t\tTYPE\t\tTARGET");
 	cur = screen.window_list;
 	do {
 		struct imwindow *imwindow = cur->data;
 
-		screen_output("%u\t\t\t%s\t\t%s\t\t%s",
+		screen_cmd_output("%u\t\t\t%s\t\t%s\t\t%s",
 			imwindow->refnum, imwindow->name,
 			win_types[imwindow->type], imwindow->target);
 
@@ -556,15 +583,19 @@ USER_COMMAND(cmd_win_prev) {
 }
 
 USER_COMMAND(cmd_win_rename) {
-	if (args != NULL)
-		imwindow_rename(cur_window(), args);
+	struct imwindow *win = cur_window();
+
+	if (args == NULL)
+		screen_cmd_output("Window %u has name \"%s\"", win->refnum, win->name);
+	else
+		imwindow_rename(win, args);
 }
 
 USER_COMMAND(cmd_win_renumber) {
 	u_int32_t num;
 
-	if (args == NULL) {
-		screen_output("This is window %u", cur_window()->refnum);
+	if (args == NULL || blank_str(args)) {
+		screen_cmd_output("This is window %u", cur_window()->refnum);
 		return;
 	}
 
@@ -631,7 +662,7 @@ USER_COMMAND(cmd_win_skip) {
 USER_COMMAND(cmd_win_swap) {
 	u_int32_t num;
 
-	if (args == NULL)
+	if (args == NULL || blank_str(args))
 		return;
 
 	if (str_to_uint(args, &num) != 0) {
@@ -691,7 +722,7 @@ USER_COMMAND(cmd_win_unskip) {
 ** History Manipulation.
 */
 
-static const struct command history_command[] = {
+static struct command history_command[] = {
 	{ "clear",		cmd_history_clear	},
 	{ "list",		cmd_history_list	},
 	{ "next",		cmd_history_next	},
@@ -711,12 +742,11 @@ USER_COMMAND(cmd_history_list) {
 	if (cur == NULL)
 		return;
 
-	screen_write(win, "Command history:");
+	screen_win_msg(win, 0, 0, 0, MSG_TYPE_CMD_OUTPUT, "Command history:");
 
 	do {
-		char *entry = (char *) cur->data;
-
-		screen_win_msg(win, 0, 0, 0, "%u: %s", i++, entry);
+		screen_win_msg(win, 0, 0, 0, MSG_TYPE_CMD_OUTPUT, "%u: %s", i++,
+			(char *) cur->data);
 		cur = cur->prev;
 	} while (cur != NULL);
 }
@@ -733,7 +763,7 @@ USER_COMMAND(cmd_history_prev) {
 ** /buddy commands
 */
 
-static const struct command buddy_command[] = {
+static struct command buddy_command[] = {
 	{ "add",			cmd_buddy_add			},
 	{ "add_group",		cmd_buddy_add_group		},
 	{ "alias",			cmd_buddy_alias			},
@@ -802,7 +832,7 @@ USER_COMMAND(cmd_buddy_block) {
 	if (buddy_add_block(acct, args, 1) == -1)
 		screen_err_msg("%s is already on your block list", args);
 	else
-		screen_status_msg("%s added to block list", args);
+		screen_cmd_output("%s added to block list", args);
 }
 
 USER_COMMAND(cmd_buddy_permit) {
@@ -814,7 +844,7 @@ USER_COMMAND(cmd_buddy_permit) {
 	if (buddy_add_permit(acct, args, 1) == -1)
 		screen_err_msg("%s is already on your permit list", args);
 	else
-		screen_status_msg("%s added to permit list", args);
+		screen_cmd_output("%s added to permit list", args);
 }
 
 USER_COMMAND(cmd_buddy_add_group) {
@@ -847,7 +877,7 @@ USER_COMMAND(cmd_buddy_alias) {
 		return;
 	}
 
-	screen_status_msg("alias: %s -> %s", buddy_name, alias);
+	screen_cmd_output("alias: %s -> %s", buddy_name, alias);
 	buddy_alias(acct, buddy, alias, 1);
 
 	/*
@@ -896,7 +926,7 @@ USER_COMMAND(cmd_buddy_privacy_mode) {
 		str_to_int(args, &mode);
 
 	mode = acct->proto->set_privacy_mode(acct, mode);
-	screen_output("Privacy mode for %s is %d", acct->username, mode);
+	screen_cmd_output("Privacy mode for %s is %d", acct->username, mode);
 }
 
 USER_COMMAND(cmd_buddy_list) {
@@ -906,14 +936,16 @@ USER_COMMAND(cmd_buddy_list) {
 	dlist_t *gcur;
 
 	gcur = pref->group_list;
-	if (gcur != NULL)
-		screen_win_msg(win, 0, 1, 1, "%s's buddy list: ", acct->username);
+	if (gcur != NULL) {
+		screen_win_msg(win, 0, 1, 1, MSG_TYPE_CMD_OUTPUT,
+			"%s's buddy list: ", acct->username);
+	}
 
 	while (gcur != NULL) {
 		struct bgroup *gr = gcur->data;
 		dlist_t *bcur = gr->member_list;
 
-		screen_win_msg(win, 0, 0, 1, "+ %s (%u/%u)",
+		screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT, "+ %s (%u/%u)",
 			gr->name, gr->num_online, gr->num_members);
 
 		while (bcur != NULL) {
@@ -950,10 +982,12 @@ USER_COMMAND(cmd_buddy_list) {
 						buddy->warn_level);
 				}
 
-				screen_win_msg(win, 0, 0, 1,
+				screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT,
 					" %so%%x %s %s", color_attr, buddy->name, buddy_status);
-			} else
-				screen_win_msg(win, 0, 0, 1, " %%Ro%%x %s", buddy->name);
+			} else {
+				screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT,
+					" %%Ro%%x %s", buddy->name);
+			}
 
 			bcur = bcur->next;
 		}
@@ -963,45 +997,39 @@ USER_COMMAND(cmd_buddy_list) {
 }
 
 USER_COMMAND(cmd_buddy_list_permit) {
-	struct imwindow *win = cur_window();
-	struct pork_acct *acct = win->owner;
+	struct pork_acct *acct = cur_window()->owner;
 	struct buddy_pref *pref = acct->buddy_pref;
 	dlist_t *cur;
 
 	cur = pref->permit_list;
 	if (cur == NULL) {
-		screen_win_msg(win, 0, 1, 1,
-			"%s's permitted users list is empty", acct->username);
+		screen_cmd_output("%s's permitted users list is empty", acct->username);
 		return;
 	}
 
-	screen_win_msg(win, 0, 1, 1,
-		"%s's permitted users list:", acct->username);
+	screen_cmd_output("%s's permitted users list:", acct->username);
 
 	while (cur != NULL) {
-		screen_output(" - %s", (char *) cur->data);
+		screen_cmd_output(" - %s", (char *) cur->data);
 		cur = cur->next;
 	}
 }
 
 USER_COMMAND(cmd_buddy_list_block) {
-	struct imwindow *win = cur_window();
-	struct pork_acct *acct = win->owner;
+	struct pork_acct *acct = cur_window()->owner;
 	struct buddy_pref *pref = acct->buddy_pref;
 	dlist_t *cur;
 
 	cur = pref->block_list;
 	if (cur == NULL) {
-		screen_win_msg(win, 0, 1, 1,
-			"%s's blocked users list is empty", acct->username);
+		screen_cmd_output("%s's blocked users list is empty", acct->username);
 		return;
 	}
 
-	screen_win_msg(win, 0, 1, 1,
-		"%s's blocked users list:", acct->username);
+	screen_cmd_output("%s's blocked users list:", acct->username);
 
 	while (cur != NULL) {
-		screen_output(" - %s", (char *) cur->data);
+		screen_cmd_output(" - %s", (char *) cur->data);
 		cur = cur->next;
 	}
 }
@@ -1032,7 +1060,7 @@ USER_COMMAND(cmd_buddy_remove_permit) {
 	if (buddy_remove_permit(acct, args, 1) != 0)
 		screen_err_msg("%s isn't on %s's permit list", args, acct->username);
 	else {
-		screen_status_msg("%s removed from %s's permit list",
+		screen_cmd_output("%s removed from %s's permit list",
 			args, acct->username);
 	}
 }
@@ -1046,7 +1074,7 @@ USER_COMMAND(cmd_buddy_unblock) {
 	if (buddy_remove_block(acct, args, 1) != 0)
 		screen_err_msg("%s isn't on %s's block list", args, acct->username);
 	else {
-		screen_status_msg("%s removed from %s's block list",
+		screen_cmd_output("%s removed from %s's block list",
 			args, acct->username);
 	}
 }
@@ -1060,7 +1088,7 @@ USER_COMMAND(cmd_buddy_remove) {
 	if (buddy_remove(acct, args, 1) != 0)
 		screen_err_msg("%s isn't on %s's buddy list", args, acct->username);
 	else {
-		screen_status_msg("%s removed from %s's buddy list",
+		screen_cmd_output("%s removed from %s's buddy list",
 			args, acct->username);
 	}
 }
@@ -1075,7 +1103,7 @@ USER_COMMAND(cmd_buddy_remove_group) {
 		screen_err_msg("The group %s does not exist on %s's buddy list",
 			args, acct->username);
 	} else {
-		screen_err_msg("The group %s was removed from %s's buddy list",
+		screen_cmd_output("The group %s was removed from %s's buddy list",
 			args, acct->username);
 	}
 }
@@ -1086,7 +1114,7 @@ USER_COMMAND(cmd_buddy_report_idle) {
 	if (acct->proto->set_report_idle == NULL)
 		return;
 
-	if (args != NULL) {
+	if (args != NULL && !blank_str(args)) {
 		u_int32_t mode;
 
 		if (str_to_uint(args, &mode) != 0) {
@@ -1097,7 +1125,7 @@ USER_COMMAND(cmd_buddy_report_idle) {
 		acct->proto->set_report_idle(acct, mode);
 	}
 
-	screen_status_msg("The reporting of idle time for %s is %s",
+	screen_cmd_output("The reporting of idle time for %s is %s",
 		acct->username, (acct->report_idle ? "enabled" : "disabled"));
 }
 
@@ -1115,12 +1143,12 @@ USER_COMMAND(cmd_buddy_seen) {
 	}
 
 	if (buddy->status != STATUS_OFFLINE) {
-		screen_output("%s is currently online", buddy->name);
+		screen_cmd_output("%s is currently online", buddy->name);
 		return;
 	}
 
-	if (buddy->last_seen == 0) {
-		screen_output("%s has never been seen online by %s",
+	if (!buddy->last_seen) {
+		screen_cmd_output("%s has never been seen online by %s",
 			args, acct->username);
 	} else {
 		char *p;
@@ -1137,7 +1165,7 @@ USER_COMMAND(cmd_buddy_seen) {
 		if (p != NULL)
 			*p = '\0';
 
-		screen_output("%s last saw %s online %s (%s ago)",
+		screen_cmd_output("%s last saw %s online %s (%s ago)",
 			acct->username, buddy->name, timestr, timebuf);
 	}
 }
@@ -1180,7 +1208,7 @@ USER_COMMAND(cmd_buddy_warn_anon) {
 ** /blist commands
 */
 
-static const struct command blist_command[] = {
+static struct command blist_command[] = {
 	{ "add_block",		cmd_blist_add_block		},
 	{ "add_permit",		cmd_blist_add_permit	},
 	{ "away",			cmd_blist_away			},
@@ -1445,7 +1473,7 @@ USER_COMMAND(cmd_blist_width) {
 ** /timer commands
 */
 
-static const struct command timer_command[] = {
+static struct command timer_command[] = {
 	{ "add",			cmd_timer_add			},
 	{ "del",			cmd_timer_del			},
 	{ "del_refnum",		cmd_timer_del_refnum	},
@@ -1495,7 +1523,7 @@ USER_COMMAND(cmd_timer_del) {
 	if (ret == -1)
 		screen_err_msg("No timer for \"%s\" was found", args);
 	else
-		screen_status_msg("Timer for \"%s\" was removed", args);
+		screen_cmd_output("Timer for \"%s\" was removed", args);
 }
 
 USER_COMMAND(cmd_timer_del_refnum) {
@@ -1514,7 +1542,7 @@ USER_COMMAND(cmd_timer_del_refnum) {
 	if (ret == -1)
 		screen_err_msg("No timer with refnum %u was found", refnum);
 	else
-		screen_status_msg("Timer with refnum %u was removed", refnum);
+		screen_cmd_output("Timer with refnum %u was removed", refnum);
 }
 
 USER_COMMAND(cmd_timer_list) {
@@ -1524,7 +1552,7 @@ USER_COMMAND(cmd_timer_list) {
 USER_COMMAND(cmd_timer_purge) {
 	if (screen.timer_list != NULL) {
 		timer_destroy(&screen.timer_list);
-		screen_status_msg("All timers have been removed");
+		screen_cmd_output("All timers have been removed");
 	}
 }
 
@@ -1532,7 +1560,7 @@ USER_COMMAND(cmd_timer_purge) {
 ** /event commands
 */
 
-static const struct command event_command[] = {
+static struct command event_command[] = {
 	{ "add",			cmd_event_add			},
 	{ "del",			cmd_event_del			},
 	{ "del_refnum",		cmd_event_del_refnum	},
@@ -1565,7 +1593,7 @@ USER_COMMAND(cmd_event_add) {
 	if (event_add(events, event_type, args, &refnum) != 0)
 		screen_err_msg("Error adding handler for %s", event_type);
 	else {
-		screen_status_msg("Event handler %s installed for %s (refnum %u)",
+		screen_cmd_output("Event handler %s installed for %s (refnum %u)",
 			args, event_type, refnum);
 	}
 }
@@ -1584,10 +1612,10 @@ USER_COMMAND(cmd_event_del) {
 	ret = event_del_type(cur_window()->owner->events, event_type, args);
 	if (ret == 0) {
 		if (args == NULL) {
-			screen_status_msg("Successfully removed handler %s for %s",
+			screen_cmd_output("Successfully removed handler %s for %s",
 				event_type, args);
 		} else
-			screen_status_msg("Successfully removed all handlers for %s", args);
+			screen_cmd_output("Successfully removed all handlers for %s", args);
 	} else {
 		if (args == NULL) {
 			screen_err_msg("Error removing handler %s for %s",
@@ -1611,7 +1639,7 @@ USER_COMMAND(cmd_event_del_refnum) {
 	if (event_del_refnum(cur_window()->owner->events, refnum) != 0)
 		screen_err_msg("Error deleting event refnum %s", args);
 	else
-		screen_status_msg("Event refnum %s was removed", args);
+		screen_cmd_output("Event refnum %s was removed", args);
 }
 
 USER_COMMAND(cmd_event_list) {
@@ -1622,11 +1650,32 @@ USER_COMMAND(cmd_event_purge) {
 	event_purge(cur_window()->owner->events);
 }
 
+/*
+** acct commands
+*/
+
+static struct command acct_command[] = {
+	{ "list",	cmd_acct_list		},
+	{ "save",	cmd_acct_save		},
+	{ "set",	cmd_acct_set		},
+};
+
+USER_COMMAND(cmd_acct_list) {
+	pork_acct_print_list();
+}
+
+USER_COMMAND(cmd_acct_save) {
+	pork_acct_save(cur_window()->owner);
+}
+
+USER_COMMAND(cmd_acct_set) {
+}
+
 /*
 ** /chat commands
 */
 
-static const struct command chat_command[] = {
+static struct command chat_command[] = {
 	{ "ban",				cmd_chat_ban			},
 	{ "ignore",				cmd_chat_ignore			},
 	{ "invite",				cmd_chat_invite			},
@@ -1634,7 +1683,6 @@ static const struct command chat_command[] = {
 	{ "kick",				cmd_chat_kick			},
 	{ "leave",				cmd_chat_leave			},
 	{ "list",				cmd_chat_list			},
-	{ "mode",				cmd_chat_mode			},
 	{ "send",				cmd_chat_send			},
 	{ "topic",				cmd_chat_topic			},
 	{ "unignore",			cmd_chat_unignore		},
@@ -1781,10 +1829,6 @@ USER_COMMAND(cmd_chat_list) {
 	chat_list(cur_window()->owner);
 }
 
-USER_COMMAND(cmd_chat_mode) {
-	/* XXX: TODO */
-}
-
 USER_COMMAND(cmd_chat_send) {
 	struct pork_acct *acct = cur_window()->owner;
 	struct imwindow *win;
@@ -1888,8 +1932,8 @@ USER_COMMAND(cmd_chat_who) {
 	chat_who(acct, args);
 }
 
-static const struct {
-	const struct command *set;
+static struct command_set {
+	struct command *set;
 	size_t elem;
 	char *type;
 } command_set[] = {
@@ -1904,16 +1948,13 @@ static const struct {
 	{	event_command,		array_elem(event_command),		"event "	},
 	{	chat_command,		array_elem(chat_command),		"chat "		},
 	{	file_command,		array_elem(file_command),		"file "		},
+	{	acct_command,		array_elem(acct_command),		"acct "		},
 };
 
 /*
 ** Main command set.
 */
 
-USER_COMMAND(cmd_acct) {
-	pork_acct_print_list();
-}
-
 USER_COMMAND(cmd_alias) {
 	char *alias;
 	char *str;
@@ -1929,7 +1970,7 @@ USER_COMMAND(cmd_alias) {
 		struct alias *lalias = alias_find(&screen.alias_hash, alias);
 
 		if (lalias != NULL) {
-			screen_output("%s is aliased to %s%s",
+			screen_cmd_output("%s is aliased to %s%s",
 				lalias->alias, lalias->orig,
 				(lalias->args != NULL ? lalias->args : ""));
 		} else
@@ -1942,7 +1983,7 @@ USER_COMMAND(cmd_alias) {
 		struct alias *lalias = alias_find(&screen.alias_hash, alias);
 
 		if (lalias != NULL) {
-			screen_status_msg("%s is aliased to %s%s",
+			screen_cmd_output("%s is aliased to %s%s",
 				lalias->alias, lalias->orig,
 				(lalias->args != NULL ? lalias->args : ""));
 
@@ -2024,9 +2065,9 @@ USER_COMMAND(cmd_bind) {
 	if (func == NULL) {
 		binding = bind_find(target_binds, key);
 		if (binding != NULL)
-			screen_output("%s is bound to %s", key_str, binding->binding);
+			screen_cmd_output("%s is bound to %s", key_str, binding->binding);
 		else
-			screen_output("%s is not bound", key_str);
+			screen_cmd_output("%s is not bound", key_str);
 
 		return;
 	}
@@ -2034,7 +2075,7 @@ USER_COMMAND(cmd_bind) {
 	bind_add(target_binds, key, func);
 	binding = bind_find(target_binds, key);
 	if (binding != NULL) {
-		screen_status_msg("%s is bound to %s", key_str, binding->binding);
+		screen_cmd_output("%s is bound to %s", key_str, binding->binding);
 		return;
 	}
 
@@ -2083,7 +2124,7 @@ USER_COMMAND(cmd_ctcp) {
 
 USER_COMMAND(cmd_echo) {
 	if (args != NULL)
-		screen_write(cur_window(), args);
+		screen_win_msg(cur_window(), 0, 0, 1, MSG_TYPE_CMD_OUTPUT, args);
 }
 
 USER_COMMAND(cmd_disconnect) {
@@ -2104,7 +2145,7 @@ USER_COMMAND(cmd_disconnect) {
 			return;
 		}
 
-		if (blank_str(args))
+		if (args != NULL && blank_str(args))
 			args = NULL;
 	}
 
@@ -2133,8 +2174,9 @@ USER_COMMAND(cmd_help) {
 		char buf[8192];
 
 		if (pork_help_get_cmds("main", buf, sizeof(buf)) != -1) {
-			screen_output("Help for the following commands is available:");
-			screen_win_msg(cur_window(), 0, 0, 1, "\t%s", buf);
+			screen_cmd_output("Help for the following commands is available:");
+			screen_win_msg(cur_window(), 0, 0, 1, MSG_TYPE_CMD_OUTPUT,
+				"\t%s", buf);
 		} else
 			screen_err_msg("Error: Can't find the help files");
 
@@ -2157,12 +2199,12 @@ USER_COMMAND(cmd_help) {
 			struct imwindow *win = cur_window();
 
 			if (pork_help_get_cmds(section, buf, sizeof(buf)) != -1) {
-				screen_write(win, " ");
+				screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT, " ");
 				strtoupper(section);
-				screen_win_msg(win, 0, 0, 1, "%%W%s COMMANDS", section);
-				screen_win_msg(win, 0, 0, 1, "\t%s", buf);
-				screen_output("Type /help %s <command> for the help text for a particular %s command.",
-					section, section);
+				screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT,
+					"%%W%s COMMANDS", section);
+				screen_win_msg(win, 0, 0, 1, MSG_TYPE_CMD_OUTPUT, "\t%s", buf);
+				screen_cmd_output("Type /help %s <command> for the help text for a particular %s command.", section, section);
 			}
 		}
 	} else {
@@ -2193,7 +2235,7 @@ USER_COMMAND(cmd_laddr) {
 		if (get_hostname(&local_addr, buf, sizeof(buf)) != 0)
 			xstrncpy(buf, "0.0.0.0", sizeof(buf));
 
-		screen_output("New connections will use the local address %s", buf);
+		screen_cmd_output("New connections will use the local address %s", buf);
 		return;
 	}
 
@@ -2202,7 +2244,7 @@ USER_COMMAND(cmd_laddr) {
 		return;
 	}
 
-	screen_output("New connections will use the local address %s", args);
+	screen_cmd_output("New connections will use the local address %s", args);
 }
 
 USER_COMMAND(cmd_lastlog) {
@@ -2262,7 +2304,8 @@ USER_COMMAND(cmd_load) {
 
 USER_COMMAND(cmd_lport) {
 	if (args == NULL) {
-		screen_output("New connections will use local port %u", ntohs(local_port));
+		screen_cmd_output("New connections will use local port %u",
+			ntohs(local_port));
 		return;
 	}
 
@@ -2272,7 +2315,7 @@ USER_COMMAND(cmd_lport) {
 	}
 
 	local_port = htons(local_port);
-	screen_output("New connections will use local port %s", args);
+	screen_cmd_output("New connections will use local port %s", args);
 }
 
 USER_COMMAND(cmd_me) {
@@ -2353,7 +2396,7 @@ USER_COMMAND(cmd_refresh) {
 
 USER_COMMAND(cmd_save) {
 	if (save_global_config() == 0)
-		screen_status_msg("Your configuration has been saved");
+		screen_cmd_output("Your configuration has been saved");
 	else
 		screen_err_msg("There was an error saving your configuration");
 }
@@ -2411,9 +2454,9 @@ USER_COMMAND(cmd_unbind) {
 	}
 
 	if (bind_remove(target_binds, c) == -1)
-		screen_err_msg("There is no binding for %s", binding);
+		screen_cmd_output("There is no binding for %s", binding);
 	else
-		screen_status_msg("Binding for %s removed", binding);
+		screen_cmd_output("Binding for %s removed", binding);
 }
 
 USER_COMMAND(cmd_unalias) {
@@ -2421,9 +2464,9 @@ USER_COMMAND(cmd_unalias) {
 		return;
 
 	if (alias_remove(&screen.alias_hash, args) == -1)
-		screen_err_msg("No such alias: %s", args);
+		screen_cmd_output("No such alias: %s", args);
 	else
-		screen_status_msg("Alias %s removed", args);
+		screen_cmd_output("Alias %s removed", args);
 }
 
 USER_COMMAND(cmd_nick) {
@@ -2599,6 +2642,13 @@ USER_COMMAND(cmd_event) {
 		run_one_command("list", CMDSET_EVENT);
 }
 
+USER_COMMAND(cmd_acct) {
+	if (args != NULL)
+		run_one_command(args, CMDSET_ACCT);
+	else
+		run_one_command("list", CMDSET_ACCT);
+}
+
 USER_COMMAND(cmd_chat) {
 	if (!cur_window()->owner->connected)
 		return;
@@ -2752,12 +2802,24 @@ static int run_one_command(char *str, u_int32_t set) {
 
 	cmd_str = strsep(&str, " \t");
 
-	cmd = bsearch(cmd_str, command_set[set].set,
-			command_set[set].elem, sizeof(struct command), cmd_compare);
+	cmd = bsearch(cmd_str, command_set[set].set, command_set[set].elem,
+			sizeof(struct command), cmd_compare);
 
 	if (cmd == NULL) {
-		screen_err_msg("Unknown %scommand: %s",
-			command_set[set].type, cmd_str);
+		struct pork_proto *proto;
+
+		if (set == CMDSET_MAIN && (proto = proto_get_name(cmd_str)) != NULL) {
+			cmd_str = strsep(&str, " \t");
+
+			cmd = bsearch(cmd_str, proto->cmd, proto->num_cmds,
+					sizeof(struct command), cmd_compare);
+
+			if (cmd == NULL)
+				screen_err_msg("Unknown %s command: %s", proto->name, cmd_str);
+		} else {
+			screen_err_msg("Unknown %scommand: %s",
+				command_set[set].type, cmd_str);
+		}
 
 		return (-1);
 	}
@@ -2771,13 +2833,13 @@ static void print_binding(void *data, void *nothing __notused) {
 	char key_name[32];
 
 	bind_get_keyname(binding->key, key_name, sizeof(key_name));
-	screen_output("%s is bound to %s", key_name, binding->binding);
+	screen_cmd_output("%s is bound to %s", key_name, binding->binding);
 }
 
 static void print_alias(void *data, void *nothing __notused) {
 	struct alias *alias = data;
 
-	screen_output("%s is aliased to %s%s",
+	screen_cmd_output("%s is aliased to %s%s",
 		alias->alias, alias->orig, (alias->args != NULL ? alias->args : ""));
 }
 
@@ -2791,15 +2853,23 @@ static int cmd_compare(const void *l, const void *r) {
 static void print_timer(void *data, void *nothing __notused) {
 	struct timer_entry *timer = data;
 
-	screen_output("[refnum: %u] %d %u %s", timer->refnum,
+	screen_cmd_output("[refnum: %u] %d %u %s", timer->refnum,
 		(int) timer->interval, timer->times, timer->command);
 }
 
 USER_COMMAND(cmd_input_find_next_cmd) {
-	struct input *input = cur_window()->input;
-	u_int16_t cur_pos = input->cur - input->prompt_len;
-	u_int16_t begin_completion_pos;
-	char *input_buf = input_get_buf_str(input);
+	u_int32_t cur_pos;
+	u_int32_t begin_completion_pos;
+	u_int32_t word_begin = 0;
+	u_int32_t end_word;
+	size_t elements = 0;
+	char *input_buf;
+	struct input *input;
+	struct command *cmd = NULL;
+
+	input = cur_window()->input;
+	cur_pos = input->cur - input->prompt_len;
+	input_buf = input_get_buf_str(input);
 
 	if (input->begin_completion <= input->prompt_len ||
 		input->begin_completion >= input->cur)
@@ -2807,7 +2877,7 @@ USER_COMMAND(cmd_input_find_next_cmd) {
 		if (cur_pos == 0) {
 			/*
 			** If you complete from the very beginning of the line,
-			** insert a '/' because we only complete commands!
+			** insert a '/' because we only complete commands.
 			*/
 
 			input_insert(input, '/');
@@ -2820,127 +2890,128 @@ USER_COMMAND(cmd_input_find_next_cmd) {
 	begin_completion_pos = input->begin_completion - input->prompt_len;
 
 	/* Only complete if the line is a command. */
-	if (input_buf[0] == '/') {
-		/* find first word... */
-		u_int32_t end_word = strcspn(&input_buf[1], " \t") + 1;
-		u_int32_t word_begin = 0;
-		u_int16_t cmdset = 0;
+	if (input_buf[0] != '/')
+		return;
 
-		if (end_word < begin_completion_pos) {
-			/* checking second word! */
-			size_t i;
+	end_word = strcspn(&input_buf[1], " \t") + 1;
 
-			for (i = 1 ; i < array_elem(command_set) ; i++) {
-				if (!strncasecmp(command_set[i].type, &input_buf[1], end_word))
-					cmdset = i;
-			}
+	if (end_word < begin_completion_pos) {
+		size_t i;
 
-			if (cmdset != 0) {
+		for (i = 1 ; i < array_elem(command_set) ; i++) {
+			if (!strncasecmp(command_set[i].type, &input_buf[1], end_word)) {
+				elements = command_set[i].elem;
+				cmd = command_set[i].set;
 				word_begin = end_word;
-				while (	input_buf[word_begin] == ' ' ||
-						input_buf[word_begin] == '\t')
-				{
-					word_begin++;
-				}
+
+				break;
 			}
-		} else {
-			/* checking first word */
-			word_begin = 1;
-			cmdset = CMDSET_MAIN;
 		}
 
-		if (word_begin > 0) {
-			int chosen = -1;
-			size_t i;
+		if (word_begin == 0) {
+			struct pork_proto *proto = cur_window()->owner->proto;
 
-			for (i = 0 ; i < command_set[cmdset].elem ; i++) {
-				const struct command *cmd_ptr;
+			if (!strncasecmp(proto->name, &input_buf[1], end_word)) {
+				elements = proto->num_cmds;
+				cmd = proto->cmd;
+				word_begin = end_word;
+			} else
+				return;
+		}
 
-				cmd_ptr = command_set[cmdset].set + i;
-				if (!strcmp(cmd_ptr->name, ""))
-					continue;
+		while (	input_buf[word_begin] == ' ' ||
+				input_buf[word_begin] == '\t')
+		{
+			word_begin++;
+		}
+	} else {
+		word_begin = 1;
+		elements = command_set[CMDSET_MAIN].elem;
+		cmd = command_set[CMDSET_MAIN].set;
+	}
 
-				if (input->cur == input->begin_completion) {
-					if (!strncasecmp(cmd_ptr->name,
-							&input_buf[word_begin], cur_pos - word_begin))
-					{
-						/*
-						** Don't 'choose' this one if
-						** it's already an exact match!
-						*/
+	if (word_begin > 0) {
+		int chosen = -1;
+		size_t i;
 
-						if (strlen(cmd_ptr->name) == cur_pos - word_begin)
-							continue;
+		for (i = 0 ; i < elements ; i++) {
+			const struct command *cmd_ptr;
 
-						/* MATCH! */
-						chosen = i;
-						i = command_set[cmdset].elem;
+			cmd_ptr = &cmd[i];
+			if (!strcmp(cmd_ptr->name, ""))
+				continue;
+
+			if (input->cur == input->begin_completion) {
+				if (!strncasecmp(cmd_ptr->name,
+						&input_buf[word_begin], cur_pos - word_begin))
+				{
+					/* Don't choose this one if it's already an exact match. */
+					if (strlen(cmd_ptr->name) == cur_pos - word_begin)
 						continue;
-					}
-				} else {
+
+					/* Match */
+					chosen = i;
+					i = elements;
+					continue;
+				}
+			} else {
+				/*
+				** We've _already_ matched one. Now get
+				** the next in the list, if possible.
+				*/
+
+				if (!strncasecmp(cmd_ptr->name,
+						&input_buf[word_begin], cur_pos - word_begin))
+				{
 					/*
-					** We've _already_ matched one. Now get
-					** the next in the list, if possible.
+					** We've found the one we've matched already. If there
+					** will be another possible, it will be the very next
+					** one in the list. Let's check that. If it doesn't
+					** match, none will.
 					*/
 
-					if (!strncasecmp(cmd_ptr->name,
-							&input_buf[word_begin], cur_pos - word_begin))
-					{
-						/*
-						** We've found the one we've matched already. If there
-						** will be another possible, it will be the very next
-						** one in the list. Let's check that. If it doesn't
-						** match, none will.
-						*/
+					i++;
+					if (i < elements) {
+						cmd_ptr = &cmd[i];
 
-						i++;
-						if (i < command_set[cmdset].elem) {
-							cmd_ptr = command_set[cmdset].set + i;
-
-							if (!strncasecmp(cmd_ptr->name,
-									&input_buf[word_begin],
-									begin_completion_pos - word_begin))
-							{
-								/*
-								** The next one matched!
-								** Let's pick it!
-								*/
-
-								chosen = i;
-							}
-
-							i = command_set[cmdset].elem;
-							continue;
+						if (!strncasecmp(cmd_ptr->name,
+								&input_buf[word_begin],
+								begin_completion_pos - word_begin))
+						{
+							chosen = i;
 						}
+
+						i = elements;
+						continue;
 					}
 				}
 			}
+		}
 
-			if (input->begin_completion != input->cur) {
-				/*
-				** Remove characters added by last match. In
-				** other words, remove characters from input->cur
-				** until input->begin_completion == input->cur.
-				*/
-				size_t num_to_remove;
+		if (input->begin_completion != input->cur) {
+			/*
+			** Remove characters added by last match. In
+			** other words, remove characters from input->cur
+			** until input->begin_completion == input->cur.
+			*/
+			size_t num_to_remove;
 
-				num_to_remove = input->cur - input->begin_completion;
+			num_to_remove = input->cur - input->begin_completion;
 
-				for (i = 0 ; i < num_to_remove ; i++)
-					input_bkspace(input);
-			}
+			for (i = 0 ; i < num_to_remove ; i++)
+				input_bkspace(input);
+		}
 
-			if (chosen != -1) {
-				/* We've found a match. */
-				const struct command *cmd_ptr;
-				u_int16_t remember_begin = input->begin_completion;
+		if (chosen != -1) {
+			/* We've found a match. */
+			const struct command *cmd_ptr;
+			u_int16_t remember_begin = input->begin_completion;
 
-				cmd_ptr = command_set[cmdset].set + chosen;
+			cmd_ptr = &cmd[chosen];
 
-				input_insert_str(input,
-					&cmd_ptr->name[begin_completion_pos - word_begin]);
-				input->begin_completion = remember_begin;
-			}
+			input_insert_str(input,
+				&cmd_ptr->name[begin_completion_pos - word_begin]);
+			input->begin_completion = remember_begin;
 		}
 	}
 }
