@@ -1,6 +1,6 @@
 /*
 ** pork_command.h - interface to commands typed by the user
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -24,6 +24,8 @@ enum {
 	CMDSET_EVENT,
 	CMDSET_CHAT,
 	CMDSET_FILE,
+	CMDSET_ACCT,
+	CMDSET_PROTO,
 };
 
 struct command {
