@@ -1,6 +1,6 @@
 /*
 ** pork_conf.c - pork's configuration parser.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -33,7 +33,7 @@
 #include <pork_misc.h>
 #include <pork_bind.h>
 #include <pork_alias.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_screen.h>
 #include <pork_command.h>
 #include <pork_conf.h>
@@ -48,8 +48,7 @@ static int pork_mkdir(const char *path) {
 		}
 	} else {
 		if (!S_ISDIR(st.st_mode)) {
-			screen_err_msg("Error: %s is not a directory. It should be.",
-				path);
+			screen_err_msg("Error: %s is not a directory", path);
 			return (-1);
 		}
 	}
@@ -464,7 +463,7 @@ int read_global_config(void) {
 
 		pork_dir = opt_get_str(OPT_PORK_DIR);
 	} else
-		snprintf(buf, sizeof(buf), "%s", pork_dir);
+		xstrncpy(buf, pork_dir, sizeof(buf));
 
 	if (pork_mkdir(buf) != 0)
 		return (-1);
