@@ -1,7 +1,6 @@
 /*
 ** pork_events.c - event handler functions.
-** Copyright (C) 2002-2004 Amber Adams <amber@ojnk.net>
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -15,6 +14,7 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <ncurses.h>
 #include <string.h>
 #include <stdarg.h>
 
@@ -23,10 +23,12 @@
 #include <pork_util.h>
 #include <pork_list.h>
 #include <pork_set.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_perl.h>
 #include <pork_command.h>
 #include <pork_events.h>
+#include <pork_imsg.h>
+#include <pork_screen.h>
 
 static u_int32_t next_refnum;
 
@@ -133,14 +135,16 @@ static void event_print_type(struct event *events, u_int32_t type) {
 	dlist_t *cur = events->e[type];
 
 	if (cur == NULL) {
-		screen_output("No %s events", events_info[type].name);
+		screen_win_msg(cur_window(), 0, 1, 1, MSG_TYPE_CMD_OUTPUT,
+			"No %s events", events_info[type].name);
 		return;
 	}
 
 	do {
 		struct event_entry *event = cur->data;
 
-		screen_output("[refnum %u] EVENT %s: %s",
+		screen_win_msg(cur_window(), 0, 1, 1, MSG_TYPE_CMD_OUTPUT,
+			"[refnum %u] EVENT %s: %s",
 			event->refnum,
 			events_info[type].name,
  			event->command);
