@@ -1,6 +1,6 @@
 /*
 ** pork_imwindow.c - interface for manipulating conversation/chat/info windows.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -22,6 +22,8 @@
 #include <pork_missing.h>
 #include <pork_util.h>
 #include <pork_list.h>
+#include <pork_imsg.h>
+#include <pork_set.h>
 #include <pork_swindow.h>
 #include <pork_imwindow.h>
 #include <pork_buddy_list.h>
@@ -32,7 +34,7 @@
 #include <pork_color.h>
 #include <pork_misc.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_format.h>
 #include <pork_chat.h>
 
@@ -175,9 +177,9 @@ void imwindow_blist_show(struct imwindow *imwindow) {
 }
 
 void imwindow_buffer_find(struct imwindow *imwindow, char *str, u_int32_t opt) {
-	screen_output("Matching lines:");
+	screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_LASTLOG, "Matching lines:");
 	swindow_print_matching(&imwindow->swindow, str, opt);
-	screen_output("End of matches");
+	screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_LASTLOG, "End of matches");
 }
 
 void imwindow_blist_hide(struct imwindow *imwindow) {
@@ -323,15 +325,11 @@ struct imwindow *imwindow_find_refnum(u_int32_t refnum) {
 	return (NULL);
 }
 
-void imwindow_send_msg(struct imwindow *win, struct pork_acct *acct, char *buf)
-{
+inline void imwindow_send_msg(struct imwindow *win) {
 	swindow_input(&win->swindow);
-	screen_win_msg(win, 1, 0, 1, "%s", buf);
 }
 
-void imwindow_recv_msg(struct imwindow *win, struct pork_acct *acct, char *buf)
-{
-	screen_win_msg(win, 1, 0, 1, "%s", buf);
+inline void imwindow_recv_msg(struct imwindow *win) {
 	if (wopt_get_bool(win->opts, WOPT_BEEP_ON_OUTPUT))
 		beep();
 }
@@ -391,11 +389,11 @@ int imwindow_bind_next_acct(struct imwindow *imwindow) {
 }
 
 inline void imwindow_scroll_up(struct imwindow *imwindow) {
-	swindow_scroll_up(&imwindow->swindow);
+	swindow_scroll_by(&imwindow->swindow, -1);
 }
 
 inline void imwindow_scroll_down(struct imwindow *imwindow) {
-	swindow_scroll_down(&imwindow->swindow);
+	swindow_scroll_by(&imwindow->swindow, 1);
 }
 
 inline void imwindow_scroll_by(struct imwindow *imwindow, int lines) {
@@ -454,6 +452,9 @@ inline int imwindow_unskip(struct imwindow *imwindow) {
 	return (ret);
 }
 
-inline int imwindow_add(struct imwindow *imwindow, chtype *msg, int timestamp) {
-	return (swindow_add(&imwindow->swindow, msg, timestamp));
+inline int imwindow_add(struct imwindow *imwindow,
+						struct imsg *imsg,
+						u_int32_t type)
+{
+	return (swindow_add(&imwindow->swindow, imsg, type));
 }
