@@ -1,6 +1,6 @@
 /*
 ** pork_io.c - I/O handler.
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -14,14 +14,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <errno.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include <pork.h>
 #include <pork_missing.h>
 #include <pork_util.h>
 #include <pork_list.h>
 #include <pork_io.h>
+#include <pork_inet.h>
 
 static dlist_t *io_list;
 
@@ -137,6 +141,29 @@ int pork_io_del_cond(void *key, u_int32_t new_cond) {
 	return (0);
 }
 
+static int pork_io_find_dead_fds(dlist_t *io_list) {
+	dlist_t *cur = io_list;
+	int bad_fd = 0;
+
+	while (cur != NULL) {
+		struct io_source *io = cur->data;
+		dlist_t *next = cur->next;
+
+		if (io->fd < 0 || sock_is_error(io->fd)) {
+			debug("fd %d is dead", io->fd);
+			if (io->callback != NULL)
+				io->callback(io->fd, IO_COND_DEAD, io->data);
+
+			pork_io_remove(cur);
+			bad_fd++;
+		}
+
+		cur = next;
+	}
+
+	return (bad_fd);
+}
+
 int pork_io_run(void) {
 	fd_set rfds;
 	fd_set wfds;
@@ -183,9 +210,17 @@ int pork_io_run(void) {
 	if (max_fd < 0)
 		return (-1);
 
+	/*
+	** If there's a bad fd in the set better find it, otherwise
+	** we're going to get into an infinite loop.
+	*/
 	ret = select(max_fd + 1, &rfds, &wfds, &xfds, &tv);
-	if (ret < 1)
+	if (ret < 1) {
+		if (ret == -1 && errno == EBADF)
+			pork_io_find_dead_fds(io_list);
+
 		return (ret);
+	}
 
 	cur = io_list;
 	while (cur != NULL) {
