@@ -1,6 +1,6 @@
 /*
 ** pork_msg.c
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -28,14 +28,16 @@
 #include <pork_set.h>
 #include <pork_format.h>
 #include <pork_events.h>
+#include <pork_imsg.h>
 #include <pork_imwindow.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_screen.h>
 #include <pork_msg.h>
 
 int pork_msg_autoreply(struct pork_acct *acct, char *dest, char *msg) {
 	struct imwindow *win;
 	char buf[4096];
+	int ret;
 
 	if (acct->proto->send_msg_auto == NULL)
 		return (-1);
@@ -44,8 +46,11 @@ int pork_msg_autoreply(struct pork_acct *acct, char *dest, char *msg) {
 		return (-1);
 
 	screen_get_query_window(acct, dest, &win);
-	fill_format_str(OPT_FORMAT_IM_SEND_AUTO, buf, sizeof(buf), acct, dest, msg);
-	imwindow_send_msg(win, acct, buf);
+	ret = fill_format_str(OPT_FORMAT_IM_SEND_AUTO, buf, sizeof(buf), acct, dest, msg);
+	if (ret < 1)
+		return (-1);
+	screen_print_str(win, buf, (size_t) ret, MSG_TYPE_PRIVMSG_SEND);
+	imwindow_send_msg(win);
 	return (0);
 }
 
@@ -102,6 +107,7 @@ int pork_recv_msg(	struct pork_acct *acct,
 	{
 		int type;
 		char buf[4096];
+		int ret;
 
 		if (autoresp)
 			type = OPT_FORMAT_IM_RECV_AUTO;
@@ -112,9 +118,12 @@ int pork_recv_msg(	struct pork_acct *acct,
 				type = OPT_FORMAT_IM_RECV;
 		}
 
-		fill_format_str(type, buf, sizeof(buf), acct, dest,
-			sender, userhost, msg);
-		imwindow_recv_msg(win, acct, buf);
+		ret = fill_format_str(type, buf, sizeof(buf), acct, dest,
+				sender, userhost, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(win, buf, (size_t) ret, MSG_TYPE_PRIVMSG_RECV);
+		imwindow_recv_msg(win);
 
 		if (acct->away_msg != NULL && !autoresp &&
 			opt_get_bool(OPT_AUTOSEND_AWAY))
@@ -164,7 +173,8 @@ int pork_set_away(struct pork_acct *acct, char *msg) {
 		}
 	}
 
-	screen_status_msg_ts("%s is now away", acct->username);
+	screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_AWAY,
+		"%s is now away", acct->username);
 	return (0);
 }
 
@@ -192,7 +202,8 @@ int pork_set_back(struct pork_acct *acct) {
 	hash_destroy(&acct->autoreply);
 	memset(&acct->autoreply, 0, sizeof(acct->autoreply));
 
-	screen_status_msg_ts("%s is no longer away", acct->username);
+	screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_BACK,
+		"%s is no longer away", acct->username);
 	return (0);
 }
 
@@ -209,6 +220,7 @@ int pork_msg_send(struct pork_acct *acct, char *dest, char *msg) {
 				struct imwindow *win;
 				char buf[4096];
 				int type;
+				int ret;
 
 				if (acct->away_msg != NULL) {
 					if (opt_get_bool(OPT_SEND_REMOVES_AWAY))
@@ -223,8 +235,11 @@ int pork_msg_send(struct pork_acct *acct, char *dest, char *msg) {
 				else
 					type = OPT_FORMAT_IM_SEND;
 
-				fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
-				imwindow_send_msg(win, acct, buf);
+				ret = fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
+				if (ret < 1)
+					return (-1);
+				screen_print_str(win, buf, (size_t) ret, MSG_TYPE_PRIVMSG_SEND);
+				imwindow_send_msg(win);
 			}
 		}
 	}
@@ -248,7 +263,8 @@ int pork_set_profile(struct pork_acct *acct, char *profile) {
 		ret = acct->proto->set_profile(acct, profile);
 
 	if (ret == 0) {
-		screen_output("Profile for %s was %s", acct->username,
+		screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_CMD_OUTPUT,
+			"Profile for %s was %s", acct->username,
 			(profile == NULL ? "cleared" : "set"));
 	}
 
@@ -265,9 +281,11 @@ int pork_set_idle_time(struct pork_acct *acct, u_int32_t seconds) {
 		return (1);
 
 	acct->proto->set_idle_time(acct, seconds);
-	time_to_str(seconds, timebuf, sizeof(timebuf));
+	time_to_str_full(seconds, timebuf, sizeof(timebuf));
+
+	screen_win_msg(cur_window(), 1, 1, 0, MSG_TYPE_CMD_OUTPUT,
+		"%s's idle time set to %s", acct->username, timebuf);
 
-	screen_output("%s's idle time set to %s", acct->username, timebuf);
 	return (0);
 }
 
@@ -279,8 +297,16 @@ int pork_send_warn(struct pork_acct *acct, char *user) {
 
 	if (!event_generate(acct->events, EVENT_SEND_WARN, user, 0, acct->refnum)) {
 		ret = acct->proto->warn(acct, user);
-		if (ret == 0)
-			screen_status_msg_ts("%s has warned %s", acct->username, user);
+		if (ret == 0) {
+			struct imwindow *win;
+
+			win = imwindow_find(acct, user);
+			if (win == NULL)
+				win = cur_window();
+
+			screen_win_msg(win, 1, 1, 0,
+				MSG_TYPE_CMD_OUTPUT, "%s has warned %s", acct->username, user);
+		}
 	}
 
 	return (ret);
@@ -294,8 +320,17 @@ int pork_send_warn_anon(struct pork_acct *acct, char *user) {
 
 	if (!event_generate(acct->events, EVENT_SEND_WARN, user, 1, acct->refnum)) {
 		ret = acct->proto->warn_anon(acct, user);
-		if (ret == 0)
-			screen_status_msg_ts("%s has warned %s anonymously", acct->username, user);
+		if (ret == 0) {
+			struct imwindow *win;
+
+			win = imwindow_find(acct, user);
+			if (win == NULL)
+				win = cur_window();
+
+			screen_win_msg(win, 0, 0, 1,
+				MSG_TYPE_CMD_OUTPUT, "%s has warned %s anonymously",
+				acct->username, user);
+		}
 	}
 
 	return (ret);
@@ -320,6 +355,7 @@ int pork_recv_action(	struct pork_acct *acct,
 		struct imwindow *win;
 		char buf[4096];
 		int type;
+		int ret;
 
 		screen_get_query_window(acct, sender, &win);
 
@@ -328,10 +364,12 @@ int pork_recv_action(	struct pork_acct *acct,
 		else
 			type = OPT_FORMAT_ACTION_RECV;
 
-		fill_format_str(type, buf, sizeof(buf), acct,
-			dest, sender, userhost, msg);
-
-		imwindow_recv_msg(win, acct, buf);
+		ret = fill_format_str(type, buf, sizeof(buf), acct,
+				dest, sender, userhost, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(win, buf, (size_t) ret, MSG_TYPE_PRIVMSG_RECV);
+		imwindow_recv_msg(win);
 	}
 
 	return (0);
@@ -351,6 +389,7 @@ int pork_action_send(struct pork_acct *acct, char *dest, char *msg) {
 		char buf[4096];
 		struct imwindow *win;
 		int type;
+		int ret;
 
 		screen_get_query_window(acct, dest, &win);
 
@@ -359,8 +398,11 @@ int pork_action_send(struct pork_acct *acct, char *dest, char *msg) {
 		else
 			type = OPT_FORMAT_ACTION_SEND;
 
-		fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
-		imwindow_send_msg(win, acct, buf);
+		ret = fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(win, buf, (size_t) ret, MSG_TYPE_PRIVMSG_SEND);
+		imwindow_send_msg(win);
 	}
 
 	return (0);
@@ -381,14 +423,18 @@ int pork_notice_send(struct pork_acct *acct, char *dest, char *msg) {
 	{
 		char buf[4096];
 		int type;
+		int ret;
 
 		if (win == screen.status_win)
 			type = OPT_FORMAT_NOTICE_SEND_STATUS;
 		else
 			type = OPT_FORMAT_NOTICE_SEND;
 
-		fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
-		imwindow_send_msg(win, acct, buf);
+		ret = fill_format_str(type, buf, sizeof(buf), acct, dest, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(win, buf, (size_t) ret, MSG_TYPE_NOTICE_SEND);
+		imwindow_send_msg(win);
 	}
 
 	return (0);
@@ -414,11 +460,14 @@ int pork_recv_notice(	struct pork_acct *acct,
 		sender, userhost, dest, msg, acct->refnum))
 	{
 		char buf[4096];
+		int ret;
 
-		fill_format_str(type, buf, sizeof(buf), acct,
-			dest, sender, userhost, msg);
-
-		imwindow_recv_msg(win, acct, buf);
+		ret = fill_format_str(type, buf, sizeof(buf), acct,
+				dest, sender, userhost, msg);
+		if (ret < 1)
+			return (-1);
+		screen_print_str(win, buf, (size_t) ret, MSG_TYPE_NOTICE_RECV);
+		imwindow_recv_msg(win);
 	}
 
 	return (0);
