@@ -1,14 +1,11 @@
 /*
 ** pork_perl.c - Perl scripting support
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** The init, execute_perl, and destroy functions were pretty much copied
 ** from the gaim perl.c source file.
 ** Copyrights there apply here.
 **
-** The execute_perl_va function is a lot more clever than the way they
-** do it, and it wasn't copied from there.
-**
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
 ** as published by the Free Software Foundation.
@@ -36,7 +33,7 @@
 #include <pork_missing.h>
 #include <pork_util.h>
 #include <pork_list.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 #include <pork_perl.h>
 #include <pork_perl_xs.h>
 
@@ -177,6 +174,7 @@ void xs_init(init_args) {
 	newXS("PORK::chat_topic", PORK_chat_topic, file);
 	newXS("PORK::chat_unignore", PORK_chat_unignore, file);
 	newXS("PORK::input_send", PORK_input_send, file);
+	newXS("PORK::input_get_data", PORK_input_get_data, file);
 }
 
 int perl_load_file(char *filename) {
@@ -229,11 +227,6 @@ int execute_perl_va(char *function, const char *fmt, va_list ap) {
 	if (SvTRUE(ERRSV)) {
 		size_t notused;
 		char *err_msg = SvPV(ERRSV, notused);
-		char *p;
-
-		p = strchr(err_msg, '\n');
-		if (p != NULL)
-			*p = '\0';
 
 		screen_err_msg("Perl error: %s", err_msg);
 		(void) POPs;
@@ -267,11 +260,6 @@ int execute_perl(char *function, char **args) {
 	if (SvTRUE(ERRSV)) {
 		size_t notused;
 		char *err_msg = SvPV(ERRSV, notused);
-		char *p;
-
-		p = strchr(err_msg, '\n');
-		if (p != NULL)
-			*p = '\0';
 
 		screen_err_msg("Perl error: %s", err_msg);
 		(void) POPs;
