@@ -1,6 +1,6 @@
 /*
 ** pork_perl_xs.c - Perl scripting support
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -44,7 +44,8 @@
 #include <pork_input.h>
 #include <pork_bind.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_imsg.h>
+#include <pork_screen_io.h>
 #include <pork_status.h>
 #include <pork_screen.h>
 #include <pork_timer.h>
@@ -119,6 +120,15 @@ XS(PORK_input_send) {
 	XSRETURN_IV(0);
 }
 
+XS(PORK_input_get_data) {
+	dXSARGS;
+
+	(void) cv;
+	(void) items;
+
+	XSRETURN_PV(input_get_buf_str(cur_window()->input));
+}
+
 XS(PORK_disconnect) {
 	struct pork_acct *acct;
 	char *reason = NULL;
@@ -210,7 +220,7 @@ XS(PORK_status_msg) {
 	if (msg == NULL)
 		XSRETURN_IV(-1);
 
-	screen_status_msg("%s", msg);
+	screen_win_msg(cur_window(), 1, 1, 1, MSG_TYPE_STATUS, "%s", msg);
 	XSRETURN_IV(0);
 }
 
@@ -279,7 +289,7 @@ XS(PORK_echo) {
 		XSRETURN_IV(-1);
 
 	msg = SvPV(ST(0), notused);
-	screen_write(cur_window(), msg);
+	screen_win_msg(cur_window(), 0, 0, 1, MSG_TYPE_CMD_OUTPUT, msg);
 	XSRETURN_IV(0);
 }
 
