@@ -1,6 +1,6 @@
 /*
 ** pork_set.c - /SET command implementation.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 ** Copyright (C) 2002-2004 Amber Adams <amber@ojnk.net>
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -29,16 +29,10 @@
 #include <pork_cstr.h>
 #include <pork_misc.h>
 #include <pork_screen.h>
-#include <pork_screen_cmd.h>
+#include <pork_screen_io.h>
 
 extern struct screen screen;
 
-static int opt_set_bool(u_int32_t opt, char *args);
-static int opt_set_char(u_int32_t opt, char *args);
-static int opt_set_int(u_int32_t opt, char *args);
-static int opt_set_str(u_int32_t opt, char *args);
-static int opt_set_color(u_int32_t opt, char *args);
-
 static int wopt_set_bool(struct imwindow *imwindow, u_int32_t opt, char *args);
 static int wopt_set_int(struct imwindow *imwindow, u_int32_t opt, char *args);
 static int wopt_set_str(struct imwindow *imwindow, u_int32_t opt, char *args);
@@ -71,8 +65,14 @@ static void wopt_changed_wordwrap(struct imwindow *imwindow);
 ** Default:		The default value of the option.
 */
 
-struct global_var global_var[] = {
-	{	"AUTO_RECONNECT",
+struct global_pref global_pref[] = {
+	{	"ACTIVITY_TYPES",
+		OPT_INT,
+		0,
+		opt_set_int,
+		NULL,
+		SET_INT(DEFAULT_ACTIVITY_TYPES)
+	},{	"AUTO_RECONNECT",
 		OPT_BOOL,
 		0,
 		opt_set_bool,
@@ -246,6 +246,12 @@ struct global_var global_var[] = {
 		opt_set_format,
 		NULL,
 		SET_STR(DEFAULT_FORMAT_CHAT_MODE),
+	},{	"FORMAT_CHAT_NICK",
+		OPT_FORMAT,
+		0,
+		opt_set_format,
+		NULL,
+		SET_STR(DEFAULT_FORMAT_CHAT_NICK),
 	},{	"FORMAT_CHAT_QUIT",
 		OPT_FORMAT,
 		0,
@@ -474,6 +480,12 @@ struct global_var global_var[] = {
 		opt_set_format,
 		NULL,
 		SET_STR(DEFAULT_FORMAT_STATUS_WARN),
+	},{	"FORMAT_TIMESTAMP",
+		OPT_FORMAT,
+		0,
+		opt_set_format,
+		NULL,
+		SET_STR(DEFAULT_FORMAT_TIMESTAMP),
 	},{	"FORMAT_WARN",
 		OPT_FORMAT,
 		0,
@@ -540,6 +552,12 @@ struct global_var global_var[] = {
 		opt_set_bool,
 		NULL,
 		SET_BOOL(DEFAULT_LOG),
+	},{	"LOG_TYPES",
+		OPT_INT,
+		0,
+		opt_set_int,
+		NULL,
+		SET_INT(DEFAULT_LOG_TYPES)
 	},{	"LOGIN_ON_STARTUP",
 		OPT_BOOL,
 		0,
@@ -773,7 +791,11 @@ struct global_var global_var[] = {
 */
 
 static struct window_var window_var[] = {
-	{	"BEEP_ON_OUTPUT",
+	{	"ACTIVITY_TYPES",
+		OPT_INT,
+		wopt_set_int,
+		NULL
+	},{	"BEEP_ON_OUTPUT",
 		OPT_BOOL,
 		wopt_set_bool,
 		NULL
@@ -785,6 +807,10 @@ static struct window_var window_var[] = {
 		OPT_BOOL,
 		wopt_set_bool,
 		wopt_changed_log
+	},{	"LOG_TYPES",
+		OPT_INT,
+		wopt_set_int,
+		NULL
 	},{	"LOGFILE",
 		OPT_STR,
 		wopt_set_str,
@@ -817,7 +843,11 @@ static struct window_var window_var[] = {
 		OPT_BOOL,
 		wopt_set_bool,
 		wopt_changed_wordwrap
-	}
+	},{	"WORDWRAP_CHAR",
+		OPT_CHAR,
+		wopt_set_char,
+		wopt_changed_wordwrap
+	},
 };
 
 /*
@@ -828,31 +858,31 @@ void wopt_print_var(struct imwindow *imwindow, int i, const char *text) {
 	switch (window_var[i].type) {
 		case OPT_BOOL:
 			screen_nocolor_msg("%s %s %s", window_var[i].name, text,
-				(imwindow->opts[i].val.b ? "TRUE" : "FALSE"));
+				(imwindow->opts[i].b ? "TRUE" : "FALSE"));
 			break;
 
 		case OPT_STR:
-			if (imwindow->opts[i].val.s != NULL &&
-				imwindow->opts[i].val.s[0] != '\0')
+			if (imwindow->opts[i].s != NULL &&
+				imwindow->opts[i].s[0] != '\0')
 			{
 				screen_nocolor_msg("%s %s \"%s\"", window_var[i].name,
-					text, imwindow->opts[i].val.s);
+					text, imwindow->opts[i].s);
 			} else
 				screen_nocolor_msg("%s is <UNSET>", window_var[i].name);
 			break;
 
 		case OPT_INT:
 			screen_nocolor_msg("%s %s %d", window_var[i].name,
-				text, imwindow->opts[i].val.i);
+				text, imwindow->opts[i].i);
 			break;
 
 		case OPT_CHAR:
-			if (isprint(imwindow->opts[i].val.c)) {
+			if (isprint(imwindow->opts[i].c)) {
 				screen_nocolor_msg("%s %s '%c'", window_var[i].name, text,
-					imwindow->opts[i].val.c);
+					imwindow->opts[i].c);
 			} else {
 				screen_nocolor_msg("%s %s 0x%x", window_var[i].name, text,
-					imwindow->opts[i].val.c);
+					imwindow->opts[i].c);
 			}
 			break;
 	}
@@ -864,27 +894,27 @@ int opt_get_val(const char *opt_name, char *buf, size_t len) {
 	if (i == -1)
 		return (-1);
 
-	switch (global_var[i].type) {
+	switch (global_pref[i].type) {
 		case OPT_BOOL:
-			snprintf(buf, len, "%d", global_var[i].val.b);
+			snprintf(buf, len, "%d", global_pref[i].val.b);
 			break;
 
 		case OPT_STR:
-			if (global_var[i].val.s == NULL)
+			if (global_pref[i].val.s == NULL)
 				return (-1);
-			snprintf(buf, len, "%s", global_var[i].val.s);
+			xstrncpy(buf, global_pref[i].val.s, len);
 			break;
 
 		case OPT_INT:
-			snprintf(buf, len, "%d", global_var[i].val.i);
+			snprintf(buf, len, "%d", global_pref[i].val.i);
 			break;
 
 		case OPT_CHAR:
-			snprintf(buf, len, "%c", global_var[i].val.c);
+			snprintf(buf, len, "%c", global_pref[i].val.c);
 			break;
 
 		case OPT_COLOR:
-			color_get_str(global_var[i].val.i, buf, len);
+			color_get_str(global_pref[i].val.i, buf, len);
 			break;
 	}
 
@@ -905,21 +935,21 @@ int wopt_get_val(	struct imwindow *imwindow,
 
 	switch (window_var[i].type) {
 		case OPT_BOOL:
-			snprintf(buf, len, "%d", imwindow->opts[i].val.b);
+			snprintf(buf, len, "%d", imwindow->opts[i].b);
 			break;
 
 		case OPT_STR:
-			if (imwindow->opts[i].val.s == NULL)
+			if (imwindow->opts[i].s == NULL)
 				return (-1);
-			snprintf(buf, len, "%s", imwindow->opts[i].val.s);
+			xstrncpy(buf, imwindow->opts[i].s, len);
 			break;
 
 		case OPT_INT:
-			snprintf(buf, len, "%d", imwindow->opts[i].val.i);
+			snprintf(buf, len, "%d", imwindow->opts[i].i);
 			break;
 
 		case OPT_CHAR:
-			snprintf(buf, len, "%c", imwindow->opts[i].val.c);
+			snprintf(buf, len, "%c", imwindow->opts[i].c);
 			break;
 	}
 
@@ -942,43 +972,43 @@ void wopt_print(struct imwindow *imwindow) {
 */
 
 void opt_print_var(int i, const char *text) {
-	switch (global_var[i].type) {
+	switch (global_pref[i].type) {
 		case OPT_BOOL:
-			screen_nocolor_msg("%s %s %s", global_var[i].name,
-				text, (global_var[i].val.b ? "TRUE" : "FALSE"));
+			screen_nocolor_msg("%s %s %s", global_pref[i].name,
+				text, (global_pref[i].val.b ? "TRUE" : "FALSE"));
 			break;
 
 		case OPT_STR:
-			if (global_var[i].val.s != NULL &&
-				global_var[i].val.s[0] != '\0')
+			if (global_pref[i].val.s != NULL &&
+				global_pref[i].val.s[0] != '\0')
 			{
-				screen_nocolor_msg("%s %s \"%s\"", global_var[i].name,
-					text, global_var[i].val.s);
+				screen_nocolor_msg("%s %s \"%s\"", global_pref[i].name,
+					text, global_pref[i].val.s);
 			} else
-				screen_nocolor_msg("%s is <UNSET>", global_var[i].name);
+				screen_nocolor_msg("%s is <UNSET>", global_pref[i].name);
 
 			break;
 
 		case OPT_INT:
-			screen_nocolor_msg("%s %s %d", global_var[i].name,
-				text, global_var[i].val.i);
+			screen_nocolor_msg("%s %s %d", global_pref[i].name,
+				text, global_pref[i].val.i);
 			break;
 
 		case OPT_CHAR:
-			if (isprint(global_var[i].val.c)) {
-				screen_nocolor_msg("%s %s '%c'", global_var[i].name,
-					text, global_var[i].val.c);
+			if (isprint(global_pref[i].val.c)) {
+				screen_nocolor_msg("%s %s '%c'", global_pref[i].name,
+					text, global_pref[i].val.c);
 			} else {
-				screen_nocolor_msg("%s %s 0x%x", global_var[i].name,
-					text, global_var[i].val.c);
+				screen_nocolor_msg("%s %s 0x%x", global_pref[i].name,
+					text, global_pref[i].val.c);
 			}
 			break;
 
 		case OPT_COLOR: {
 			char buf[128];
 
-			color_get_str(global_var[i].val.i, buf, sizeof(buf));
-			screen_nocolor_msg("%s %s %s", global_var[i].name, text, buf);
+			color_get_str(global_pref[i].val.i, buf, sizeof(buf));
+			screen_nocolor_msg("%s %s %s", global_pref[i].name, text, buf);
 			break;
 		}
 	}
@@ -989,7 +1019,7 @@ static void scrollbuf_len_update(void) {
 
 	scrollbuf_len = opt_get_int(OPT_SCROLLBUF_LEN);
 	if (scrollbuf_len < cur_window()->swindow.rows)
-		global_var[OPT_SCROLLBUF_LEN].val.i = cur_window()->swindow.rows;
+		global_pref[OPT_SCROLLBUF_LEN].val.i = cur_window()->swindow.rows;
 }
 
 /*
@@ -999,49 +1029,49 @@ static void scrollbuf_len_update(void) {
 void opt_print(void) {
 	size_t i;
 
-	for (i = 0 ; i < array_elem(global_var) ; i++)
+	for (i = 0 ; i < array_elem(global_pref) ; i++)
 		opt_print_var(i, "is set to");
 }
 
 void opt_write(FILE *fp) {
 	size_t i;
 
-	for (i = 0 ; i < array_elem(global_var) ; i++) {
-		switch (global_var[i].type) {
+	for (i = 0 ; i < array_elem(global_pref) ; i++) {
+		switch (global_pref[i].type) {
 			case OPT_BOOL:
-				fprintf(fp, "set %s %s\n", global_var[i].name,
-					(global_var[i].val.b ? "TRUE" : "FALSE"));
+				fprintf(fp, "set %s %s\n", global_pref[i].name,
+					(global_pref[i].val.b ? "TRUE" : "FALSE"));
 				break;
 
 			case OPT_STR:
-				if (global_var[i].val.s != NULL &&
-					global_var[i].val.s[0] != '\0')
+				if (global_pref[i].val.s != NULL &&
+					global_pref[i].val.s[0] != '\0')
 				{
-					fprintf(fp, "set %s %s\n", global_var[i].name,
-						global_var[i].val.s);
+					fprintf(fp, "set %s %s\n", global_pref[i].name,
+						global_pref[i].val.s);
 				}
 				break;
 
 			case OPT_INT:
-				fprintf(fp, "set %s %d\n", global_var[i].name,
-					global_var[i].val.i);
+				fprintf(fp, "set %s %d\n", global_pref[i].name,
+					global_pref[i].val.i);
 				break;
 
 			case OPT_CHAR:
-				if (isprint(global_var[i].val.c)) {
-					fprintf(fp, "set %s %c\n", global_var[i].name,
-						global_var[i].val.c);
+				if (isprint(global_pref[i].val.c)) {
+					fprintf(fp, "set %s %c\n", global_pref[i].name,
+						global_pref[i].val.c);
 				} else {
-					fprintf(fp, "set %s 0x%x\n", global_var[i].name,
-						global_var[i].val.c);
+					fprintf(fp, "set %s 0x%x\n", global_pref[i].name,
+						global_pref[i].val.c);
 				}
 				break;
 
 			case OPT_COLOR: {
 				char buf[128];
 
-				color_get_str(global_var[i].val.i, buf, sizeof(buf));
-				fprintf(fp, "set %s %s\n", global_var[i].name, buf);
+				color_get_str(global_pref[i].val.i, buf, sizeof(buf));
+				fprintf(fp, "set %s %s\n", global_pref[i].name, buf);
 				break;
 			}
 		}
@@ -1050,7 +1080,7 @@ void opt_write(FILE *fp) {
 
 static int opt_compare(const void *l, const void *r) {
 	const char *str = l;
-	const struct global_var *gvar = r;
+	const struct global_pref *gvar = r;
 
 	return (strcasecmp(str, gvar->name));
 }
@@ -1068,17 +1098,17 @@ static int wopt_compare(const void *l, const void *r) {
 */
 
 int opt_find(const char *name) {
-	struct global_var *gvar;
+	struct global_pref *gvar;
 	u_int32_t offset;
 
-	gvar = bsearch(name, global_var, array_elem(global_var),
-				sizeof(struct global_var), opt_compare);
+	gvar = bsearch(name, global_pref, array_elem(global_pref),
+				sizeof(struct global_pref), opt_compare);
 
 	if (gvar == NULL)
 		return (-1);
 
-	offset = (long) gvar - (long) &global_var[0];
-	return (offset / sizeof(struct global_var));
+	offset = (long) gvar - (long) &global_pref[0];
+	return (offset / sizeof(struct global_pref));
 }
 
 /*
@@ -1120,7 +1150,7 @@ static void wopt_changed_log(struct imwindow *imwindow) {
 			swindow_end_log(&imwindow->swindow);
 		else {
 			if (swindow_set_log(&imwindow->swindow) == -1)
-				imwindow->opts[WOPT_LOG].val.b = 0;
+				imwindow->opts[WOPT_LOG].b = 0;
 		}
 	}
 }
@@ -1142,7 +1172,7 @@ static void wopt_changed_scrollbuf_len(struct imwindow *imwindow) {
 
 	new_len = wopt_get_int(imwindow->opts, WOPT_SCROLLBUF_LEN);
 	if (new_len < imwindow->swindow.rows) {
-		imwindow->opts[WOPT_SCROLLBUF_LEN].val.i = imwindow->swindow.rows;
+		imwindow->opts[WOPT_SCROLLBUF_LEN].i = imwindow->swindow.rows;
 		imwindow->swindow.scrollbuf_max = imwindow->swindow.rows;
 	} else
 		imwindow->swindow.scrollbuf_max = new_len;
@@ -1197,20 +1227,23 @@ void wopt_init(struct imwindow *imwindow, const char *target) {
 	char buf[NUSER_LEN + 256];
 	char *pork_dir;
 	char *p;
-	struct window_opts *wopt = imwindow->opts;
+	pref_val_t *wopt = imwindow->opts;
 
-	memset(wopt, 0, sizeof(struct window_opts) * WOPT_NUM_OPTS);
+	memset(wopt, 0, sizeof(pref_val_t) * WOPT_NUM_OPTS);
 
-	wopt[WOPT_BEEP_ON_OUTPUT].val.b = opt_get_bool(OPT_BEEP_ON_OUTPUT);
-	wopt[WOPT_HISTORY_LEN].val.i = opt_get_int(OPT_HISTORY_LEN);
-	wopt[WOPT_LOG].val.b = opt_get_bool(OPT_LOG);
-	wopt[WOPT_PRIVATE_INPUT].val.b = opt_get_bool(OPT_PRIVATE_INPUT);
-	wopt[WOPT_SCROLL_ON_INPUT].val.b = opt_get_bool(OPT_SCROLL_ON_INPUT);
-	wopt[WOPT_SCROLL_ON_OUTPUT].val.b = opt_get_bool(OPT_SCROLL_ON_OUTPUT);
-	wopt[WOPT_SCROLLBUF_LEN].val.i = opt_get_int(OPT_SCROLLBUF_LEN);
-	wopt[WOPT_SHOW_BLIST].val.b = opt_get_bool(OPT_SHOW_BLIST);
-	wopt[WOPT_TIMESTAMP].val.b = opt_get_bool(OPT_TIMESTAMP);
-	wopt[WOPT_WORDWRAP].val.b = opt_get_bool(OPT_WORDWRAP);
+	wopt[WOPT_ACTIVITY_TYPES].i = opt_get_int(OPT_ACTIVITY_TYPES);
+	wopt[WOPT_BEEP_ON_OUTPUT].b = opt_get_bool(OPT_BEEP_ON_OUTPUT);
+	wopt[WOPT_HISTORY_LEN].i = opt_get_int(OPT_HISTORY_LEN);
+	wopt[WOPT_LOG].b = opt_get_bool(OPT_LOG);
+	wopt[WOPT_LOG_TYPES].i = opt_get_int(OPT_LOG_TYPES);
+	wopt[WOPT_PRIVATE_INPUT].b = opt_get_bool(OPT_PRIVATE_INPUT);
+	wopt[WOPT_SCROLL_ON_INPUT].b = opt_get_bool(OPT_SCROLL_ON_INPUT);
+	wopt[WOPT_SCROLL_ON_OUTPUT].b = opt_get_bool(OPT_SCROLL_ON_OUTPUT);
+	wopt[WOPT_SCROLLBUF_LEN].i = opt_get_int(OPT_SCROLLBUF_LEN);
+	wopt[WOPT_SHOW_BLIST].b = opt_get_bool(OPT_SHOW_BLIST);
+	wopt[WOPT_TIMESTAMP].b = opt_get_bool(OPT_TIMESTAMP);
+	wopt[WOPT_WORDWRAP].b = opt_get_bool(OPT_WORDWRAP);
+	wopt[WOPT_WORDWRAP_CHAR].c = opt_get_char(OPT_WORDWRAP_CHAR);
 
 	normalize(nnick, target, sizeof(nnick));
 	while ((p = strchr(nnick, '/')) != NULL)
@@ -1221,7 +1254,7 @@ void wopt_init(struct imwindow *imwindow, const char *target) {
 	snprintf(buf, sizeof(buf), "%s/%s/logs/%s.log",
 		pork_dir, imwindow->owner->username, nnick);
 
-	wopt[WOPT_LOGFILE].val.s = xstrdup(buf);
+	wopt[WOPT_LOGFILE].s = xstrdup(buf);
 }
 
 /*
@@ -1253,24 +1286,24 @@ static int opt_tristate(char *args) {
 	return (-1);
 }
 
-static int opt_set_bool(u_int32_t opt, char *args) {
+int opt_set_bool(u_int32_t opt, char *args) {
 	int val = opt_tristate(args);
 
 	if (val == -1)
 		return (-1);
 
 	if (val != 2)
-		global_var[opt].val.b = val;
+		global_pref[opt].val.b = val;
 	else
-		global_var[opt].val.b = !global_var[opt].val.b;
+		global_pref[opt].val.b = !global_pref[opt].val.b;
 
-	if (global_var[opt].updated != NULL)
-		global_var[opt].updated();
+	if (global_pref[opt].updated != NULL)
+		global_pref[opt].updated();
 
 	return (0);
 }
 
-static int opt_set_char(u_int32_t opt, char *args) {
+int opt_set_char(u_int32_t opt, char *args) {
 	if (args == NULL || *args == '\0')
 		return (-1);
 
@@ -1278,19 +1311,19 @@ static int opt_set_char(u_int32_t opt, char *args) {
 		u_int32_t temp;
 
 		if (str_to_uint(args, &temp) == -1 || temp > 0xff)
-			global_var[opt].val.c = *args;
+			global_pref[opt].val.c = *args;
 		else
-			global_var[opt].val.c = temp;
+			global_pref[opt].val.c = temp;
 	} else
-		global_var[opt].val.c = *args;
+		global_pref[opt].val.c = *args;
 
-	if (global_var[opt].updated != NULL)
-		global_var[opt].updated();
+	if (global_pref[opt].updated != NULL)
+		global_pref[opt].updated();
 
 	return (0);
 }
 
-static int opt_set_int(u_int32_t opt, char *args) {
+int opt_set_int(u_int32_t opt, char *args) {
 	u_int32_t num;
 
 	if (args == NULL)
@@ -1299,33 +1332,33 @@ static int opt_set_int(u_int32_t opt, char *args) {
 	if (str_to_uint(args, &num) != 0)
 		return (-1);
 
-	global_var[opt].val.i = num;
+	global_pref[opt].val.i = num;
 
-	if (global_var[opt].updated != NULL)
-		global_var[opt].updated();
+	if (global_pref[opt].updated != NULL)
+		global_pref[opt].updated();
 
 	return (0);
 }
 
-static int opt_set_str(u_int32_t opt, char *args) {
-	if (global_var[opt].dynamic == 1)
-		free(global_var[opt].val.s);
+int opt_set_str(u_int32_t opt, char *args) {
+	if (global_pref[opt].dynamic == 1)
+		free(global_pref[opt].val.s);
 
 	if (args != NULL) {
-		global_var[opt].val.s = xstrdup(args);
-		global_var[opt].dynamic = 1;
+		global_pref[opt].val.s = xstrdup(args);
+		global_pref[opt].dynamic = 1;
 	} else {
-		global_var[opt].val.s = NULL;
-		global_var[opt].dynamic = 0;
+		global_pref[opt].val.s = NULL;
+		global_pref[opt].dynamic = 0;
 	}
 
-	if (global_var[opt].updated != NULL)
-		global_var[opt].updated();
+	if (global_pref[opt].updated != NULL)
+		global_pref[opt].updated();
 
 	return (0);
 }
 
-static int opt_set_color(u_int32_t opt, char *args) {
+int opt_set_color(u_int32_t opt, char *args) {
 	attr_t attr = 0;
 
 	if (*args != '%') {
@@ -1336,22 +1369,22 @@ static int opt_set_color(u_int32_t opt, char *args) {
 		chtype ch[4];
 
 		snprintf(buf, sizeof(buf), "%s ", args);
-		if (plaintext_to_cstr(buf, ch, array_elem(ch)) != 1)
+		if (plaintext_to_cstr(ch, array_elem(ch), buf, NULL) != 1)
 			return (-1);
 
 		attr = ch[0] & A_ATTRIBUTES;
 	}
 
-	global_var[opt].val.i = attr;
+	global_pref[opt].val.i = attr;
 
-	if (global_var[opt].updated != NULL)
-		global_var[opt].updated();
+	if (global_pref[opt].updated != NULL)
+		global_pref[opt].updated();
 
 	return (0);
 }
 
 int opt_set(u_int32_t opt, char *args) {
-	struct global_var *var = &global_var[opt];
+	struct global_pref *var = &global_pref[opt];
 
 	return (var->set(opt, args));
 }
@@ -1363,9 +1396,9 @@ static int wopt_set_bool(struct imwindow *imwindow, u_int32_t opt, char *args) {
 		return (-1);
 
 	if (val != 2)
-		imwindow->opts[opt].val.b = val;
+		imwindow->opts[opt].b = val;
 	else
-		imwindow->opts[opt].val.b = !imwindow->opts[opt].val.b;
+		imwindow->opts[opt].b = !imwindow->opts[opt].b;
 
 	if (window_var[opt].updated != NULL)
 		window_var[opt].updated(imwindow);
@@ -1381,11 +1414,11 @@ static int wopt_set_char(struct imwindow *imwindow, u_int32_t opt, char *args) {
 		u_int32_t temp;
 
 		if (str_to_uint(args, &temp) == -1 || temp > 0xff)
-			imwindow->opts[opt].val.c = *args;
+			imwindow->opts[opt].c = *args;
 		else
-			imwindow->opts[opt].val.c = temp;
+			imwindow->opts[opt].c = temp;
 	} else
-		imwindow->opts[opt].val.c = *args;
+		imwindow->opts[opt].c = *args;
 
 	if (window_var[opt].updated != NULL)
 		window_var[opt].updated(imwindow);
@@ -1402,7 +1435,7 @@ static int wopt_set_int(struct imwindow *imwindow, u_int32_t opt, char *args) {
 	if (str_to_uint(args, &num) != 0)
 		return (-1);
 
-	imwindow->opts[opt].val.i = num;
+	imwindow->opts[opt].i = num;
 
 	if (window_var[opt].updated != NULL)
 		window_var[opt].updated(imwindow);
@@ -1411,12 +1444,12 @@ static int wopt_set_int(struct imwindow *imwindow, u_int32_t opt, char *args) {
 }
 
 static int wopt_set_str(struct imwindow *imwindow, u_int32_t opt, char *args) {
-	free(imwindow->opts[opt].val.s);
+	free(imwindow->opts[opt].s);
 
 	if (args != NULL)
-		imwindow->opts[opt].val.s = xstrdup(args);
+		imwindow->opts[opt].s = xstrdup(args);
 	else
-		imwindow->opts[opt].val.s = NULL;
+		imwindow->opts[opt].s = NULL;
 
 	if (window_var[opt].updated != NULL)
 		window_var[opt].updated(imwindow);
@@ -1433,12 +1466,12 @@ inline int wopt_set(struct imwindow *imwindow, u_int32_t opt, char *args) {
 void opt_destroy(void) {
 	size_t i;
 
-	for (i = 0 ; i < array_elem(global_var) ; i++) {
-		if (global_var[i].dynamic)
-			free(global_var[i].val.s);
+	for (i = 0 ; i < array_elem(global_pref) ; i++) {
+		if (global_pref[i].dynamic)
+			free(global_pref[i].val.s);
 	}
 }
 
 void wopt_destroy(struct imwindow *imwindow) {
-	free(imwindow->opts[WOPT_LOGFILE].val.s);
+	free(imwindow->opts[WOPT_LOGFILE].s);
 }
