@@ -1,6 +1,6 @@
 /*
 ** pork_set.h - /SET command implementation.
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -10,24 +10,24 @@
 #ifndef __PORK_SET_H
 #define __PORK_SET_H
 
-#define OPT_INT				0
-#define OPT_CHAR			1
-#define OPT_STR				2
-/* OPT_FORMAT is functionally identical to OPT_STRING */
-#define OPT_FORMAT			2
-#define OPT_BOOL			3
-#define OPT_COLOR			4
-
 #define opt_set_format opt_set_str
 
-#include "pork_set_defaults.h"
+enum {
+	OPT_INT 		= 0,
+	OPT_CHAR 		= 1,
+	OPT_STR 		= 2,
+	OPT_FORMAT 		= 2,
+	OPT_BOOL 		= 3,
+	OPT_COLOR 		= 4
+};
 
 /*
 ** Global options.
 */
 
 enum {
-	OPT_AUTO_RECONNECT = 0,
+	OPT_ACTIVITY_TYPES = 0,
+	OPT_AUTO_RECONNECT,
 	OPT_AUTO_REJOIN,
 	OPT_AUTOSEND_AWAY,
 	OPT_BANNER,
@@ -56,6 +56,7 @@ enum {
 	OPT_FORMAT_CHAT_KICK,
 	OPT_FORMAT_CHAT_LEAVE,
 	OPT_FORMAT_CHAT_MODE,
+	OPT_FORMAT_CHAT_NICK,
 	OPT_FORMAT_CHAT_QUIT,
 	OPT_FORMAT_CHAT_RECV,
 	OPT_FORMAT_CHAT_RECV_ACTION,
@@ -94,6 +95,7 @@ enum {
 	OPT_FORMAT_STATUS_TIMESTAMP,
 	OPT_FORMAT_STATUS_TYPING,
 	OPT_FORMAT_STATUS_WARN,
+	OPT_FORMAT_TIMESTAMP,
 	OPT_FORMAT_WARN,
 	OPT_FORMAT_WHOIS_AWAY,
 	OPT_FORMAT_WHOIS_IDLE,
@@ -105,6 +107,7 @@ enum {
 	OPT_HISTORY_LEN,
 	OPT_IDLE_AFTER,
 	OPT_LOG,
+	OPT_LOG_TYPES,
 	OPT_LOGIN_ON_STARTUP,
 	OPT_OUTGOING_MSG_FONT,
 	OPT_OUTGOING_MSG_FONT_BGCOLOR,
@@ -147,16 +150,16 @@ enum {
 
 #define OPT_FORMAT_OFFSET OPT_FORMAT_ACTION_RECV
 
-extern struct global_var global_var[OPT_NUM_OPTS];
-
 /*
 ** Per-window options.
 */
 
 enum {
-	WOPT_BEEP_ON_OUTPUT = 0,
+	WOPT_ACTIVITY_TYPES = 0,
+	WOPT_BEEP_ON_OUTPUT,
 	WOPT_HISTORY_LEN,
 	WOPT_LOG,
+	WOPT_LOG_TYPES,
 	WOPT_LOGFILE,
 	WOPT_PRIVATE_INPUT,
 	WOPT_SCROLL_ON_INPUT,
@@ -165,32 +168,26 @@ enum {
 	WOPT_SHOW_BLIST,
 	WOPT_TIMESTAMP,
 	WOPT_WORDWRAP,
-	WOPT_NUM_OPTS
+	WOPT_WORDWRAP_CHAR,
+	WOPT_NUM_OPTS,
 };
 
 struct imwindow;
 
-struct global_var {
+typedef union {
+	u_int32_t i;
+	u_int32_t b;
+	char c;
+	char *s;
+} pref_val_t;
+
+struct global_pref {
 	char *name;
 	u_int32_t type:31;
 	u_int32_t dynamic:1;
 	int (*set)(u_int32_t, char *);
 	void (*updated)(void);
-	union {
-		u_int32_t i;
-		u_int32_t b;
-		char c;
-		char *s;
-	} val;
-};
-
-struct window_opts {
-	union {
-		u_int32_t i;
-		u_int32_t b;
-		char c;
-		char *s;
-	} val;
+	pref_val_t val;
 };
 
 struct window_var {
@@ -200,6 +197,14 @@ struct window_var {
 	void (*updated)(struct imwindow *);
 };
 
+extern struct global_pref global_pref[OPT_NUM_OPTS];
+
+int opt_set_bool(u_int32_t opt, char *args);
+int opt_set_char(u_int32_t opt, char *args);
+int opt_set_int(u_int32_t opt, char *args);
+int opt_set_str(u_int32_t opt, char *args);
+int opt_set_color(u_int32_t opt, char *args);
+
 #define SET_STR(x)	{ .s = (x) }
 #define SET_INT(x)	{ .i = (x) }
 #define SET_CHAR(x)	{ .c = (x) }
@@ -235,15 +240,17 @@ int wopt_get_val(	struct imwindow *imwindow,
 ** possible ways to tell it "inline this" I used.
 */
 
-#define opt_get_int(opt) (global_var[(opt)].val.i)
-#define opt_get_color(opt) (global_var[(opt)].val.i)
-#define opt_get_bool(opt) (global_var[(opt)].val.b)
-#define opt_get_char(opt) (global_var[(opt)].val.c)
-#define opt_get_str(opt) (global_var[(opt)].val.s)
+#define opt_get_int(opt) (global_pref[(opt)].val.i)
+#define opt_get_color(opt) (global_pref[(opt)].val.i)
+#define opt_get_bool(opt) (global_pref[(opt)].val.b)
+#define opt_get_char(opt) (global_pref[(opt)].val.c)
+#define opt_get_str(opt) (global_pref[(opt)].val.s)
 
-#define wopt_get_int(wopt, opt) ((wopt)[(opt)].val.i)
-#define wopt_get_str(wopt, opt) ((wopt)[(opt)].val.s)
-#define wopt_get_char(wopt, opt) ((wopt)[(opt)].val.c)
-#define wopt_get_bool(wopt, opt) ((wopt)[(opt)].val.b)
+#define wopt_get_int(wopt, opt) ((wopt)[(opt)].i)
+#define wopt_get_str(wopt, opt) ((wopt)[(opt)].s)
+#define wopt_get_char(wopt, opt) ((wopt)[(opt)].c)
+#define wopt_get_bool(wopt, opt) ((wopt)[(opt)].b)
+
+#include "pork_set_defaults.h"
 
 #endif
