@@ -1,6 +1,6 @@
 /*
 ** pork_swindow.c - an ncurses scrolling window widget
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** I've tried to keep this code independent from the rest of the program
 ** so that it might be useful outside of this particular program.
@@ -30,173 +30,11 @@
 #include <pork_swindow.h>
 #include <pork_cstr.h>
 #include <pork_misc.h>
-#include <pork_screen_cmd.h>
+#include <pork_imsg.h>
+#include <pork_screen_io.h>
 
 static void swindow_scroll(struct swindow *swindow, int n);
 
-
-static u_int32_t imsg_wordwrapped_lines(struct swindow *swindow,
-										struct imsg *imsg)
-{
-	u_int32_t len = MSG_LEN(swindow, imsg);
-	chtype *ch = MSG_TEXT(swindow, imsg);
-	chtype *end = &ch[len - 1];
-	u_int32_t lines = 0;
-	int add = 0;
-	chtype cont_char = opt_get_char(OPT_WORDWRAP_CHAR);
-
-	if (len <= swindow->cols)
-		return (1);
-
-	if (swindow->cols == 1)
-		return (len);
-
-	while (*ch != 0) {
-		chtype *p;
-
-		p = &ch[swindow->cols - 1 - add];
-		if (p >= end) {
-			++lines;
-			break;
-		}
-
-		/*
-		** If the last character on current line or the first character
-		** on the next line is a space, there's no need to break the line
-		*/
-		if (chtype_get(p[0]) != ' ' && chtype_get(p[1]) != ' ') {
-			chtype *temp = p;
-
-			/* Find the last space, if any */
-			do {
-				temp--;
-				if (temp <= ch) {
-					/*
-					** No spaces on the line, just split it at the
-					** last column
-					*/
-					goto out;
-				}
-			} while (chtype_get(*temp) != ' ');
-			/* Found a space, split the line after the space. */
-			p = temp;
-		}
-out:
-		if (cont_char != 0)
-			add = 1;
-
-		ch = &p[1];
-		lines++;
-	}
-
-	return (lines);
-}
-
-/*
-** Compute the number of rows the message will occupy
-** on the screen.
-*/
-
-static inline u_int32_t imsg_lines(struct swindow *swindow, struct imsg *imsg) {
-	u_int32_t len = MSG_LEN(swindow, imsg);
-
-	if (len <= swindow->cols)
-		return (1);
-
-	if (!swindow->wordwrap)
-		return ((len / swindow->cols) + (len % swindow->cols != 0));
-
-	return (imsg_wordwrapped_lines(swindow, imsg));
-}
-
-/*
-** Tag a message with a timestamp and store its length and the resulting
-** tagged string in the imsg struct.
-*/
-
-static struct imsg *imsg_new(	struct swindow *swindow,
-								chtype *msg,
-								u_int32_t timestamp)
-{
-	char timebuf[16];
-	u_int32_t len;
-	chtype *msg_text;
-	struct imsg *imsg;
-	char *p;
-	chtype *ch;
-	chtype *c;
-
-	if (timestamp) {
-		time_t cur_time;
-		struct tm *tm;
-
-		time(&cur_time);
-		tm = localtime(&cur_time);
-		snprintf(timebuf, sizeof(timebuf), "[%02d:%02d] ",
-			tm->tm_hour, tm->tm_min);
-	} else
-		timebuf[0] = '\0';
-
-	len = strlen(timebuf) + cstrlen(msg) + 1;
-	msg_text = xmalloc(len * sizeof(chtype));
-
-	ch = msg_text;
-	p = timebuf;
-	while (*p != '\0')
-		*ch++ = (chtype) *p++;
-
-	c = msg;
-	while (*c != 0)
-		*ch++ = *c++;
-	*ch = 0;
-
-	imsg = xmalloc(sizeof(*imsg));
-	imsg->text = msg_text;
-	imsg->serial = swindow->serial++;
-	imsg->timestamp = timestamp;
-	imsg->len = len - 1;
-	imsg->lines = imsg_lines(swindow, imsg);
-
-	return (imsg);
-}
-
-/*
-** Return a pointer to the first character of the nth
-** line of the message, given the current screen width.
-*/
-
-static chtype *imsg_partial(struct swindow *swindow,
-							struct imsg *imsg,
-							u_int32_t n)
-{
-	chtype *text = MSG_TEXT(swindow, imsg);
-	u_int32_t offset;
-	static chtype nul_ch = 0;
-
-	--n;
-	if (n == 0)
-		return (text);
-
-	offset = n * swindow->cols;
-
-	/*
-	** offset can end up larger than the length
-	** of the string when the screen has been cleared.
-	*/
-
-	if (offset < imsg->len - (text - imsg->text)) {
-		text += offset;
-		return (text);
-	}
-
-	/*
-	** If the offset is greater than the string's length
-	** the caller doesn't want to write anything to the screen
-	*/
-
-	return (&nul_ch);
-}
-
 static int swindow_print_msg_wr(struct swindow *swindow,
 								struct imsg *imsg,
 								u_int32_t y,
@@ -204,13 +42,13 @@ static int swindow_print_msg_wr(struct swindow *swindow,
 								u_int32_t firstline,
 								u_int32_t lastline)
 {
-	u_int32_t len = MSG_LEN(swindow, imsg);
-	chtype *ch = MSG_TEXT(swindow, imsg);
+	u_int32_t len = imsg->len;
+	chtype *ch = imsg->text;
 	u_int32_t lines = 0;
 	u_int32_t lines_printed = 0;
 	chtype *end = &ch[len - 1];
 	int add = 0;
-	chtype cont_char = (chtype) opt_get_char(OPT_WORDWRAP_CHAR);
+	chtype cont_char = (chtype) swindow->wordwrap_char;
 
 	if (swindow->cols == 1)
 		cont_char = 0;
@@ -274,18 +112,10 @@ static int swindow_print_msg(	struct swindow *swindow,
 		return (-1);
 
 	msg = imsg_partial(swindow, imsg, firstline);
-	if (lastline >= imsg->lines)
-		mvwputstr(swindow->win, y, x, msg);
-	else {
-		u_int32_t max_len;
 
-		if (lastline == firstline)
-			max_len = swindow->cols;
-		else
-			max_len = (lastline - firstline) * swindow->cols;
-
-		mvwputnstr(swindow->win, y, x, msg, max_len);
-	}
+	mvwputnstr(swindow->win, y, x, msg,
+		min(swindow->rows * swindow->cols,
+			(lastline - firstline + 1) * swindow->cols));
 
 	return (0);
 }
@@ -298,7 +128,7 @@ int swindow_init(	struct swindow *swindow,
 					WINDOW *win,
 					u_int32_t rows,
 					u_int32_t cols,
-					struct window_opts *wopt)
+					pref_val_t *wopt)
 {
 	set_default_win_opts(win);
 
@@ -315,8 +145,11 @@ int swindow_init(	struct swindow *swindow,
 	swindow->scroll_on_output = wopt_get_bool(wopt, WOPT_SCROLL_ON_OUTPUT);
 	swindow->timestamp = wopt_get_bool(wopt, WOPT_TIMESTAMP);
 	swindow->wordwrap = wopt_get_bool(wopt, WOPT_WORDWRAP);
+	swindow->wordwrap_char = wopt_get_char(wopt, WOPT_WORDWRAP_CHAR);
 	swindow->logged = wopt_get_bool(wopt, WOPT_LOG);
 	swindow->logfile = wopt_get_str(wopt, WOPT_LOGFILE);
+	swindow->activity_type = wopt_get_int(wopt, WOPT_ACTIVITY_TYPES);
+	swindow->log_type = wopt_get_int(wopt, WOPT_LOG_TYPES);
 
 	if (swindow->scrollbuf_max < rows)
 		swindow->scrollbuf_max = rows;
@@ -344,7 +177,6 @@ static inline void swindow_scroll(struct swindow *swindow, int n) {
 /*
 ** Prune the scroll buffer so that the number of total
 ** messages is not greater than swindow->scrollbuf_max.
-**
 */
 
 void swindow_prune(struct swindow *swindow) {
@@ -379,7 +211,7 @@ void swindow_prune(struct swindow *swindow) {
 
 /*
 ** Adjust the swindow->scrollbuf_top pointer so that it's pointing to
-** the line that's "n" lines from the current top of the screen.
+** the line that's "n" lines up from the current top of the screen.
 */
 
 static void swindow_adjust_top(struct swindow *swindow, u_int32_t n) {
@@ -455,19 +287,8 @@ static void swindow_recalculate(struct swindow *swindow,
 	** before the resize to stay there.
 	*/
 
-	if (swindow->top_hidden != 0 && imsg_top->lines != old_top) {
-		u_int32_t offset = old_cols * swindow->top_hidden;
-
-		/*
-		** If the dimensions of the window are the same, timestamps have
-		** been toggled.
-		*/
-
-		if (old_cols == swindow->cols && old_rows == swindow->rows)
-			offset -= TIME_HEADER_LEN;
-
-		swindow->top_hidden = offset / swindow->cols;
-	}
+	if (swindow->top_hidden != 0 && imsg_top->lines != old_top)
+		swindow->top_hidden = (old_cols * swindow->top_hidden) / swindow->cols;
 }
 
 /*
@@ -502,7 +323,6 @@ void swindow_resize(struct swindow *swindow,
 
 	swindow_recalculate(swindow, old_rows, old_cols);
 	swindow_redraw(swindow);
-	swindow->dirty = 1;
 }
 
 inline int swindow_refresh(struct swindow *swindow) {
@@ -535,13 +355,15 @@ void swindow_redraw(struct swindow *swindow) {
 	if (swindow->top_hidden != 0) {
 		struct imsg *imsg = cur->data;
 
-		swindow_print_msg(swindow, imsg, 0, 0, swindow->top_hidden + 1, -1);
 		curs_pos += imsg->lines - swindow->top_hidden;
 		if (curs_pos > swindow->rows) {
 			swindow->bottom_blank = 0;
 			swindow->bottom_hidden = curs_pos - swindow->rows;
 			swindow->scrollbuf_bot = cur;
-		}
+			swindow_print_msg(swindow, imsg, 0, 0, swindow->top_hidden + 1,
+				curs_pos + swindow->rows);
+		} else
+			swindow_print_msg(swindow, imsg, 0, 0, swindow->top_hidden + 1, -1);
 
 		cur = cur->prev;
 	}
@@ -586,9 +408,7 @@ void swindow_redraw(struct swindow *swindow) {
 ** display.
 */
 
-int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp) {
-	struct imsg *imsg;
-	chtype *msg_text;
+int swindow_add(struct swindow *swindow, struct imsg *imsg, u_int32_t msgtype) {
 	u_int32_t msg_line_start = 1;
 	dlist_t *old_head = swindow->scrollbuf;
 	int y_pos;
@@ -606,20 +426,17 @@ int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp) {
 		swindow_scroll_to_end(swindow);
 	}
 
-	imsg = imsg_new(swindow, msg, timestamp);
-	msg_text = MSG_TEXT(swindow, imsg);
-
-	if (swindow->logged) {
+	if (swindow->logged && (swindow->log_type & msgtype)) {
 		struct iovec wvec[2];
-		u_int32_t msg_len = MSG_LEN(swindow, imsg);
-		char *plaintext = cstr_to_plaintext(msg_text, msg_len);
+		char *plaintext;
 
+		plaintext = cstr_to_plaintext(imsg->text, imsg->len);
 		wvec[0].iov_base = plaintext;
-		wvec[0].iov_len = msg_len;
+		wvec[0].iov_len = imsg->len;
 		wvec[1].iov_base = "\n";
 		wvec[1].iov_len = 1;
 
-		if (writev(swindow->log_fd, wvec, 2) != (int) msg_len + 1) {
+		if (writev(swindow->log_fd, wvec, 2) != (int) imsg->len + 1) {
 			screen_err_msg("Error writing logfile: %s",
 				strerror(errno));
 		}
@@ -649,7 +466,8 @@ int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp) {
 
 	if (old_head != swindow->scrollbuf_bot) {
 		swindow->held += imsg->lines;
-		swindow->activity = 1;
+		if (swindow->activity_type & msgtype)
+			swindow->activity = 1;
 		return (0);
 	}
 
@@ -689,7 +507,7 @@ int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp) {
 	swindow_print_msg(swindow, imsg, y_pos, 0, msg_line_start, -1);
 	swindow->dirty = 1;
 
-	if (!swindow->visible)
+	if (!swindow->visible && (swindow->activity_type & msgtype))
 		swindow->activity = 1;
 
 	/*
@@ -702,11 +520,7 @@ int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp) {
 	return (0);
 }
 
-/*
-** This is a wrapper for swindow_add() used for adding
-** a message to a window that was typed by the user.
-*/
-
+/* Called when a message sent by a user is written to a window. */
 inline int swindow_input(struct swindow *swindow) {
 	/*
 	** If the window is scrolled up, and the scroll on
@@ -724,130 +538,6 @@ inline int swindow_input(struct swindow *swindow) {
 	return (0);
 }
 
-/*
-** Scroll up one line.
-*/
-
-u_int32_t swindow_scroll_up(struct swindow *swindow) {
-	struct imsg *imsg;
-	u_int32_t top_line;
-
-	/*
-	** If it's already at the top, it can't scroll anymore.
-	*/
-
-	if (swindow->scrollbuf_top == NULL)
-		return (0);
-
-	if (swindow->scrollbuf_top->next == NULL && swindow->top_hidden == 0)
-		return (0);
-
-	/*
-	** Adjust the pointer to the bottom line.
-	*/
-
-	imsg = swindow->scrollbuf_bot->data;
-
-	/*
-	** If there are blank lines at the bottom of the screen,
-	** there'll be one less after scrolling up a line, and the
-	** pointer to the bottom on the screen won't change.
-	*/
-
-	if (swindow->bottom_blank > 0)
-		swindow->bottom_blank--;
-	else {
-		/*
-		** Adjust the pointer to the bottom line on the screen.
-		*/
-
-		swindow->bottom_hidden++;
-
-		if (swindow->bottom_hidden == imsg->lines) {
-			swindow->bottom_hidden = 0;
-			swindow->scrollbuf_bot = swindow->scrollbuf_bot->next;
-		}
-	}
-
-	swindow_scroll(swindow, -1);
-
-	if (swindow->top_hidden > 0) {
-		imsg = swindow->scrollbuf_top->data;
-		swindow->top_hidden--;
-	} else {
-		swindow->scrollbuf_top = swindow->scrollbuf_top->next;
-		imsg = swindow->scrollbuf_top->data;
-		swindow->top_hidden = imsg->lines - 1;
-	}
-
-	top_line = swindow->top_hidden + 1;
-	swindow_print_msg(swindow, imsg, 0, 0, top_line, top_line);
-	return (1);
-}
-
-/*
-** Scroll the screen down by one line.
-*/
-
-u_int32_t swindow_scroll_down(struct swindow *swindow) {
-	struct imsg *imsg;
-	u_int32_t bottom_line;
-
-	/*
-	** If it's already at the bottom, it can't scroll anymore.
-	*/
-
-	if (swindow->scrollbuf_bot == swindow->scrollbuf &&
-		swindow->bottom_hidden == 0)
-	{
-		swindow->held = 0;
-		return (0);
-	}
-
-	/*
-	** If there are blank lines at the bottom of the screen,
-	** the window can't scroll down.
-	*/
-
-	if (swindow->bottom_blank > 0) {
-		swindow->held = 0;
-		return (0);
-	}
-
-	/*
-	** Adjust the pointer to the top line on the screen.
-	*/
-
-	imsg = swindow->scrollbuf_top->data;
-	swindow->top_hidden++;
-
-	if (swindow->top_hidden == imsg->lines) {
-		swindow->top_hidden = 0;
-		swindow->scrollbuf_top = swindow->scrollbuf_top->prev;
-	}
-
-	swindow_scroll(swindow, 1);
-
-	if (swindow->bottom_hidden > 0) {
-		imsg = swindow->scrollbuf_bot->data;
-		swindow->bottom_hidden--;
-	} else {
-		swindow->scrollbuf_bot = swindow->scrollbuf_bot->prev;
-		imsg = swindow->scrollbuf_bot->data;
-		swindow->bottom_hidden = imsg->lines - 1;
-	}
-
-	bottom_line = imsg->lines - swindow->bottom_hidden;
-
-	swindow_print_msg(swindow, imsg, swindow->rows - 1, 0,
-		bottom_line, bottom_line);
-
-	if (swindow->held > 0)
-		swindow->held--;
-
-	return (1);
-}
-
 /*
 ** Scroll to the end of the scroll buffer (the bottom line).
 */
@@ -912,30 +602,102 @@ void swindow_scroll_to_start(struct swindow *swindow) {
 	swindow_redraw(swindow);
 }
 
-/*
-** This should probably be rewritten to adjust the top (and top_hidden)
-** scrollbuf variables and redrawing instead of scrolling up/down one
-** line at a time if "lines" is more than a few lines..
-*/
+static u_int32_t swindow_scroll_down_by(struct swindow *swindow, u_int32_t lines) {
+	u_int32_t i;
 
-void swindow_scroll_by(struct swindow *swindow, int lines) {
-	if (lines < 0) {
-		int i;
+	/*
+	** Adjust the pointer to the top line on the screen.
+	*/
 
-		lines = -lines;
+	for (i = 0 ; i < lines ; i++) {
+		struct imsg *imsg;
 
-		for (i = 0 ; i < lines ; i++) {
-			if (swindow_scroll_up(swindow) == 0)
-				break;
+		if (swindow->scrollbuf_bot->prev == NULL &&
+			swindow->bottom_hidden == 0)
+		{
+			swindow->held = 0;
+			break;
 		}
-	} else {
-		int i;
 
-		for (i = 0 ; i < lines ; i++) {
-			if (swindow_scroll_down(swindow) == 0)
-				break;
+		imsg = swindow->scrollbuf_top->data;
+
+		if (++swindow->top_hidden == imsg->lines) {
+			swindow->top_hidden = 0;
+			if (swindow->scrollbuf_top->prev == NULL)
+				return (1);
+			swindow->scrollbuf_top = swindow->scrollbuf_top->prev;
+		}
+
+		if (swindow->bottom_hidden > 0)
+			swindow->bottom_hidden--;
+		else {
+			swindow->scrollbuf_bot = swindow->scrollbuf_bot->prev;
+			imsg = swindow->scrollbuf_bot->data;
+			swindow->bottom_hidden = imsg->lines - 1;
 		}
 	}
+
+	return (i);
+}
+
+static u_int32_t swindow_scroll_up_by(struct swindow *swindow, u_int32_t lines) {
+	dlist_t *cur;
+
+	if (swindow->top_hidden == 0 && swindow->scrollbuf_top->next == NULL)
+		return (0);
+
+	if (swindow->top_hidden >= lines) {
+		swindow->top_hidden -= lines;
+		return (1);
+	}
+
+	lines -= swindow->top_hidden;
+	swindow->top_hidden = 0;
+
+	cur = swindow->scrollbuf_top->next;
+	while (lines > 0 && cur != NULL) {
+		struct imsg *msg = cur->data;
+
+		if (msg->lines >= lines) {
+			swindow->scrollbuf_top = cur;
+			swindow->top_hidden = msg->lines - lines;
+			return (1);
+		}
+
+		lines -= msg->lines;
+		cur = cur->next;
+	}
+
+	if (lines > 0)
+		swindow->scrollbuf_top = swindow->scrollbuf_end;
+
+	return (1);
+}
+
+int swindow_scroll_by(struct swindow *swindow, int lines) {
+	/* Can't scroll if there are less (or equal to) lines than rows */
+	if (swindow->scrollbuf_lines > swindow->rows) {
+		int ret;
+
+		if (lines < 0)
+			ret = swindow_scroll_up_by(swindow, -lines);
+		else
+			ret = swindow_scroll_down_by(swindow, lines);
+
+		if (ret > 0) {
+			wclear(swindow->win);
+			swindow_redraw(swindow);
+			swindow_refresh(swindow);
+
+			/* If the window's visible, a delay is not acceptable. */
+			if (swindow->visible)
+				screen_doupdate();
+
+			return (1);
+		}
+	}
+
+	return (0);
 }
 
 /*
@@ -968,7 +730,7 @@ int swindow_print_matching(	struct swindow *swindow,
 		struct imsg *imsg = cur->data;
 		char *buf;
 
-		buf = cstr_to_plaintext(MSG_TEXT(swindow, imsg), MSG_LEN(swindow, imsg));
+		buf = cstr_to_plaintext(imsg->text, imsg->len);
 		if (buf != NULL) {
 			if (regexec(&preg, buf, 0, NULL, 0) == 0)
 				match_list = dlist_add_head(match_list, imsg);
@@ -988,7 +750,7 @@ int swindow_print_matching(	struct swindow *swindow,
 	while (cur != NULL) {
 		dlist_t *next = cur->next;
 
-		swindow_add(swindow, MSG_TEXT(swindow, (struct imsg *) cur->data), 0);
+		swindow_add(swindow, imsg_copy(swindow, cur->data), MSG_TYPE_LASTLOG);
 		free(cur);
 		cur = next;
 	}
@@ -1001,17 +763,7 @@ int swindow_print_matching(	struct swindow *swindow,
 */
 
 inline void swindow_set_timestamp(struct swindow *swindow, u_int32_t value) {
-	/*
-	** If nothing's changed, there's nothing to do.
-	*/
-
-	if (value == swindow->timestamp)
-		return;
-
 	swindow->timestamp = value;
-	swindow_recalculate(swindow, swindow->rows, swindow->cols);
-	wclear(swindow->win);
-	swindow_redraw(swindow);
 }
 
 inline void swindow_set_wordwrap(struct swindow *swindow, u_int32_t value) {
@@ -1093,7 +845,7 @@ int swindow_set_log(struct swindow *swindow) {
 	u_int32_t len;
 
 	if (swindow->logfile == NULL) {
-		screen_err_msg("No logfile has been specified for this window. Specify one with /win set logfile");
+		screen_err_msg("No logfile has been specified for this window");
 		return (-1);
 	}
 
