@@ -1,6 +1,6 @@
 /*
 ** pork_swindow.h - an ncurses scrolling window widget
-** Copyright (C) 2002-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2002-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -10,39 +10,13 @@
 #ifndef __PORK_SWINDOW_H
 #define __PORK_SWINDOW_H
 
-#define TIME_HEADER_LEN 8 /* "[NN:NN] " */
-
-#define MSG_LEN(swin, imsg) \
-	((!(swin)->timestamp && (imsg)->timestamp) ? (imsg)->len - TIME_HEADER_LEN : (imsg)->len)
-
-#define MSG_TEXT(swin, imsg) \
-	((!(swin)->timestamp && (imsg)->timestamp) ? (imsg)->text + TIME_HEADER_LEN : (imsg)->text)
-
 #define SWINDOW_FIND_ICASE		0x01
 #define SWINDOW_FIND_BASIC		0x02
 
-struct window_opts;
-
-struct imsg {
-	chtype *text;
-	u_int32_t serial:31;
-	u_int32_t timestamp:1;
-	u_int32_t len;
-	u_int32_t lines;
-};
+struct imsg;
 
 struct swindow {
 	WINDOW *win;
-
-	u_int32_t visible:1;
-	u_int32_t dirty:1;
-	u_int32_t activity:1;
-	u_int32_t beep_on_output:1;
-	u_int32_t scroll_on_input:1;
-	u_int32_t scroll_on_output:1;
-	u_int32_t timestamp:1;
-	u_int32_t logged:1;
-	u_int32_t wordwrap:1;
 	u_int32_t serial;
 
 	u_int32_t rows;
@@ -60,7 +34,6 @@ struct swindow {
 	** at the top or bottom of the screen aren't
 	** currently being displayed on screen.
 	*/
-
 	u_int32_t top_hidden;
 	u_int32_t bottom_hidden;
 
@@ -72,18 +45,33 @@ struct swindow {
 	dlist_t *scrollbuf_top;
 	dlist_t *scrollbuf_bot;
 
+	/* window-specific preferences */
+	u_int32_t activity_type;
+	u_int32_t log_type;
+
 	char *logfile;
 	int log_fd;
+
+	char wordwrap_char;
+	u_int32_t visible:1;
+	u_int32_t dirty:1;
+	u_int32_t activity:1;
+	u_int32_t beep_on_output:1;
+	u_int32_t scroll_on_input:1;
+	u_int32_t scroll_on_output:1;
+	u_int32_t timestamp:1;
+	u_int32_t logged:1;
+	u_int32_t wordwrap:1;
 };
 
 int swindow_init(	struct swindow *swindow,
 					WINDOW *win,
 					u_int32_t rows,
 					u_int32_t cols,
-					struct window_opts *wopt);
+					pref_val_t *wopt);
 
 int swindow_destroy(struct swindow *swindow);
-int swindow_add(struct swindow *swindow, chtype *msg, u_int32_t timestamp);
+int swindow_add(struct swindow *swindow, struct imsg *imsg, u_int32_t type);
 int swindow_input(struct swindow *swindow);
 void swindow_redraw(struct swindow *swindow);
 void swindow_clear(struct swindow *swindow);
@@ -108,8 +96,6 @@ void swindow_prune(struct swindow *swindow);
 
 void swindow_scroll_to_end(struct swindow *swindow);
 void swindow_scroll_to_start(struct swindow *swindow);
-void swindow_scroll_by(struct swindow *swindow, int lines);
-u_int32_t swindow_scroll_up(struct swindow *swindow);
-u_int32_t swindow_scroll_down(struct swindow *swindow);
+int swindow_scroll_by(struct swindow *swindow, int lines);
 
 #endif
