@@ -1,6 +1,6 @@
 /*
 ** file_transfer.c
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -30,7 +30,9 @@
 #include <pork_inet.h>
 #include <pork_acct.h>
 #include <pork_io.h>
-#include <pork_screen_cmd.h>
+#include <pork_imsg.h>
+#include <pork_screen.h>
+#include <pork_screen_io.h>
 #include <pork_set.h>
 #include <pork_proto.h>
 #include <pork_format.h>
@@ -124,6 +126,8 @@ int transfer_bind_listen_sock(struct file_transfer *xfer, int sock) {
 }
 
 int transfer_resume(struct file_transfer *xfer, off_t offset) {
+	int ret;
+
 	if (xfer->fp == NULL)
 		return (-1);
 
@@ -131,7 +135,12 @@ int transfer_resume(struct file_transfer *xfer, off_t offset) {
 		return (-1);
 
 	xfer->start_offset = offset;
-	return (fseek(xfer->fp, offset, SEEK_SET));
+
+	ret = fseek(xfer->fp, offset, SEEK_SET);
+	if (ret < 0)
+		debug("fseek: %lld %s", offset, strerror(errno));
+
+	return (ret);
 }
 
 inline struct file_transfer *transfer_find_refnum(	struct pork_acct *acct,
@@ -184,6 +193,7 @@ void transfer_send_data(int fd __notused, u_int32_t cond __notused, void *data)
 
 	ret = fread(buf, 1, sizeof(buf), xfer->fp);
 	if (ret < 0) {
+		debug("fread: %s", strerror(errno));
 		transfer_lost(xfer);
 		return;
 	}
@@ -193,7 +203,7 @@ void transfer_send_data(int fd __notused, u_int32_t cond __notused, void *data)
 
 	sent = sock_write(xfer->sock, buf, ret);
 	if (sent != ret) {
-		screen_err_msg("Error sending file %s: %s",
+		screen_err_msg("Error sending data for file %s: %s",
 			xfer->fname_local, strerror(errno));
 		transfer_lost(xfer);
 		return;
@@ -225,7 +235,8 @@ inline double transfer_avg_speed(struct file_transfer *xfer) {
 }
 
 int transfer_recv_complete(struct file_transfer *xfer) {
-	char buf[1024];
+	char buf[4096];
+	int ret;
 
 	if (xfer->acct->proto->file_recv_complete != NULL &&
 		xfer->acct->proto->file_recv_complete(xfer) == -1)
@@ -238,14 +249,19 @@ int transfer_recv_complete(struct file_transfer *xfer) {
 		return (-1);
 	}
 
-	fill_format_str(OPT_FORMAT_FILE_RECV_COMPLETE, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
+	ret = fill_format_str(OPT_FORMAT_FILE_RECV_COMPLETE, buf,
+			sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_FIN);
+	}
 
 	return (transfer_destroy(xfer));
 }
 
 int transfer_send_complete(struct file_transfer *xfer) {
-	char buf[1024];
+	char buf[4096];
+	int ret;
 
 	if (xfer->acct->proto->file_send_complete != NULL &&
 		xfer->acct->proto->file_send_complete(xfer) == -1)
@@ -258,14 +274,18 @@ int transfer_send_complete(struct file_transfer *xfer) {
 		return (-1);
 	}
 
-	fill_format_str(OPT_FORMAT_FILE_SEND_COMPLETE, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
+	ret = fill_format_str(OPT_FORMAT_FILE_SEND_COMPLETE, buf, sizeof(buf),
+			xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_FIN);
+	}
 
 	return (transfer_destroy(xfer));
 }
 
 static int transfer_find_filename(struct file_transfer *xfer, char *filename) {
-	char buf[1024];
+	char buf[4096];
 	int ret;
 	int fd;
 	FILE *fp;
@@ -367,6 +387,47 @@ int transfer_get(struct file_transfer *xfer, char *filename) {
 	return (0);
 }
 
+u_int32_t transfer_get_all(struct pork_acct *acct) {
+	dlist_t *cur;
+	u_int32_t successes = 0;
+
+	cur = acct->transfer_list;
+	while (cur != NULL) {
+		struct file_transfer *xfer = cur->data;
+		dlist_t *next = cur->next;
+
+		if (xfer->type == TRANSFER_DIR_RECV &&
+			xfer->status == TRANSFER_STATUS_WAITING)
+		{
+			if (transfer_get(xfer, xfer->fname_local) != -1)
+				successes++;
+		}
+
+		cur = next;
+	}
+
+	return (successes);
+}
+
+u_int32_t transfer_abort_all(struct pork_acct *acct, u_int32_t type) {
+	dlist_t *cur;
+	u_int32_t successes = 0;
+
+	cur = acct->transfer_list;
+	while (cur != NULL) {
+		struct file_transfer *xfer = cur->data;
+		dlist_t *next = cur->next;
+
+		if (type == TRANSFER_DIR_ANY || xfer->type == type) {
+			transfer_abort(xfer);
+			successes++;
+		}
+		cur = next;
+	}
+
+	return (successes);
+}
+
 struct file_transfer *transfer_find(struct pork_acct *acct,
 									char *peer,
 									char *file)
@@ -402,6 +463,7 @@ int transfer_send(struct pork_acct *acct, char *dest, char *filename) {
 
 	fp = fopen(buf, "r");
 	if (fp == NULL) {
+		debug("fopen: %s: %s", buf, strerror(errno));
 		screen_err_msg("Error: Cannot open %s for reading", buf);
 		return (-1);
 	}
@@ -480,69 +542,109 @@ inline char *transfer_get_remote_hostname(struct file_transfer *xfer) {
 }
 
 int transfer_recv_accepted(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
 
-	if (xfer->start_offset != 0)
-		fill_format_str(OPT_FORMAT_FILE_RECV_RESUME, buf, sizeof(buf), xfer);
-	else
-		fill_format_str(OPT_FORMAT_FILE_RECV_ACCEPT, buf, sizeof(buf), xfer);
+	if (xfer->start_offset != 0) {
+		ret = fill_format_str(OPT_FORMAT_FILE_RECV_RESUME, buf, sizeof(buf),
+				xfer);
+	} else {
+		ret = fill_format_str(OPT_FORMAT_FILE_RECV_ACCEPT, buf, sizeof(buf),
+				xfer);
+	}
+
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_STATUS);
+	}
 
-	screen_status_msg_ts("%s", buf);
 	return (0);
 }
 
 int transfer_request_recv(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
+
+	ret = fill_format_str(OPT_FORMAT_FILE_RECV_ASK, buf, sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_STATUS);
+	}
 
-	fill_format_str(OPT_FORMAT_FILE_RECV_ASK, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
 	return (0);
 }
 
 int transfer_request_send(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
+
+	ret = fill_format_str(OPT_FORMAT_FILE_SEND_ASK, buf, sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_STATUS);
+	}
 
-	fill_format_str(OPT_FORMAT_FILE_SEND_ASK, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
 	return (0);
 }
 
 int transfer_send_accepted(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
 
-	if (xfer->start_offset != 0)
-		fill_format_str(OPT_FORMAT_FILE_SEND_RESUME, buf, sizeof(buf), xfer);
-	else
-		fill_format_str(OPT_FORMAT_FILE_SEND_ACCEPT, buf, sizeof(buf), xfer);
+	if (xfer->start_offset != 0) {
+		ret = fill_format_str(OPT_FORMAT_FILE_SEND_RESUME, buf, sizeof(buf),
+				xfer);
+	} else {
+		ret = fill_format_str(OPT_FORMAT_FILE_SEND_ACCEPT, buf, sizeof(buf),
+				xfer);
+	}
+
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_START);
+	}
 
-	screen_status_msg_ts("%s", buf);
 	xfer->status = TRANSFER_STATUS_ACTIVE;
 	return (0);
 }
 
 int transfer_lost(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
 
-	fill_format_str(OPT_FORMAT_FILE_LOST, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
+	ret = fill_format_str(OPT_FORMAT_FILE_LOST, buf, sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_STATUS);
+	}
 
 	return (transfer_abort(xfer));
 }
 
 int transfer_cancel_local(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
 
-	fill_format_str(OPT_FORMAT_FILE_CANCEL_LOCAL, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
+	ret = fill_format_str(OPT_FORMAT_FILE_CANCEL_LOCAL, buf, sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_FIN);
+	}
 
 	return (transfer_abort(xfer));
 }
 
 int transfer_cancel_remote(struct file_transfer *xfer) {
-	char buf[512];
+	char buf[4096];
+	int ret;
+
+	ret = fill_format_str(OPT_FORMAT_FILE_CANCEL_REMOTE,
+		buf, sizeof(buf), xfer);
+	if (ret > 0) {
+		screen_print_str(cur_window(), buf, (size_t) ret,
+			MSG_TYPE_FILE_XFER_FIN);
+	}
 
-	fill_format_str(OPT_FORMAT_FILE_CANCEL_LOCAL, buf, sizeof(buf), xfer);
-	screen_status_msg_ts("%s", buf);
 	return (transfer_abort(xfer));
 }
 
