@@ -1,6 +1,6 @@
 /*
 ** pork_transfer.h
-** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
+** Copyright (C) 2003-2005 Ryan McCabe <ryan@numb.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -21,6 +21,7 @@ struct pork_acct;
 enum {
 	TRANSFER_DIR_SEND,
 	TRANSFER_DIR_RECV,
+	TRANSFER_DIR_ANY,
 };
 
 enum {
@@ -84,6 +85,8 @@ inline char *transfer_get_remote_hostname(struct file_transfer *xfer);
 int transfer_cancel_local(struct file_transfer *xfer);
 int transfer_cancel_remote(struct file_transfer *xfer);
 int transfer_lost(struct file_transfer *xfer);
+u_int32_t transfer_get_all(struct pork_acct *acct);
+u_int32_t transfer_abort_all(struct pork_acct *acct, u_int32_t type);
 
 struct file_transfer *transfer_new(	struct pork_acct *acct,
 									char *peer,
